
class MAIN_FORM_EVENT_HANDLER

inherit
   FORM_MANAGER_CLIENT
   DATABASE_DEMO_CONSTANTS
   STANDARD_EVENT_HANDLER
      redefine
         handle_frm_open_event, handle_lst_select_event, 
	 handle_ctl_select_event
      end
   
creation   
   make
   
feature {ANY} --creation
   
   make( application_database : RECORD_DATABASE ) is
      do
	 create details_form.from_form_id( Details_form )
	 create sort_form.from_form_id( Sort_form )
	 create sort_form_event_handler
	 sort_form.set_event_handler( sort_form_event_handler )
	 create current_record
	 create sort_comparison.make
	 database := application_database
      end
   
   
feature {ANY} -- event handlers

   handle_frm_open_event( event : EVENT ) is
      do
	 populate_main_list
         form_manager.active_form.draw
         last_event_was_handled := True
      end
   
   handle_lst_select_event( event : EVENT ) is
      --handle selection of an item in the list.  In this case, we 
      --will be editing the details of the selected item
      do
	 show_details_for_record_at_index( event.data_listselect_selection )
	 last_event_was_handled := False
      end
   
   handle_ctl_select_event( event : EVENT ) is
      --there is only one button on the main form, so this simple 
      --creates a new database entry and edits it using the details dialog
      do
	 inspect event.data_ctlselect_controlid
	 when Main_form_new_button then
	    create_new_record
	    last_event_was_handled := True
	 when Main_form_sort_button then
	    sort_database
	    last_event_was_handled := True
	 else
	    last_event_was_handled := False
	 end
      end
   
feature {ANY} -- application features
   
      alpha_code_from_control_id( control_id : INTEGER ) : INTEGER is
      do
	 inspect control_id
	 when Details_pushbutton_a then
	    Result := 0
	 when Details_pushbutton_b then
	    Result := 1
	 when Details_pushbutton_c then
	    Result := 2
	 when Details_pushbutton_d then
	    Result := 3
	 else
	    Result := 0
	 end
      end
   
   control_id_from_alpha_code( alpha_code : INTEGER_16 ) : INTEGER_16 is
      do
	 inspect alpha_code
	 when 0 then
	    Result := Details_pushbutton_a
	 when 1 then
	    Result := Details_pushbutton_b
	 when 2 then
	    Result := Details_pushbutton_c
	 when 3 then
	    Result := Details_pushbutton_d
	 else
	    Result := Details_pushbutton_a
	 end
      end
   
   character_from_alpha_code( alpha_code : INTEGER_16 ) : CHARACTER is
      do
	 inspect alpha_code
	 when 0 then
	    Result := 'A'
	 when 1 then
	    Result := 'B'
	 when 2 then
	    Result := 'C'
	 when 3 then
	    Result := 'D'
	 else
	    Result := '?'
	 end
      end
   
   set_controls_to_record( form : FORM ) is
      require      
	 current_record /= Void
      do
	 form .set_control_group_selection_by_id( 1, control_id_from_alpha_code( current_record.alpha_code.to_integer ) )
	 form.field_from_id( Details_quantity_field ).set_text_to_string( current_record.quantity.to_string )
	 form.field_from_id( Details_name_field ).set_text_to_string( current_record.name )
      end	 
   
   set_record_to_controls( form : FORM ) is
      do
	 current_record.set_alpha_code( alpha_code_from_control_id( form.object_id_from_index( form.control_group_selection( 1 ) ) ).to_character )
	 current_record.set_quantity( form.field_from_id( Details_quantity_field ).contents.to_integer.to_integer_16 )
	 current_record.set_name( form.field_from_id( Details_name_field ).contents )
      end
   
   last_details_dialog_result : INTEGER_16
   
   edit_current_record is
      do
	 --draw once to properly save background
	 details_form.draw
	 set_controls_to_record( details_form )
	 --now draw again to show our changes to the controls correctly
	 details_form.draw
	 details_form.do_dialog
	 last_details_dialog_result := details_form.last_dialog_result
	 if last_details_dialog_result = Details_button_ok then
	    set_record_to_controls( details_form )
	 end
      end
   
   create_new_record is
	 --create a new record in the database and edit it
      do
	 current_record.set_name( "bob")
	 current_record.set_quantity( 42 )
	 current_record.set_alpha_code( '%/1/' )
	 edit_current_record
	 if details_form.last_dialog_result = Details_button_ok then
	    database.new_typed_record( database.max_record_index, current_record );--database.number_of_records + 1, current_record )
	    populate_main_list
	 end
      end
   
   set_current_record_to_record_at_index( index : INTEGER_16 ) is
	 --set the "current record" to the database record with the 
	 --given index
      do
	 database.set_typed_record_from_index( index, current_record )
      end
   
   set_record_at_index_to_current_record( index : INTEGER_16 ) is
	 --set the record at the given index to the current record
      do
	 database.write_typed_record_to_index( index, current_record )
      end
   
   show_details_for_record_at_index( index : INTEGER_16 ) is
	 --edit the record at the given index
      do
	 set_current_record_to_record_at_index( index )
	 --edit the record
	 edit_current_record
	 --if "ok" selected, set the database record at the index to 
	 --the current record
	 if last_details_dialog_result = Details_button_ok then
	    set_record_at_index_to_current_record( index )
	    populate_main_list
	 elseif last_details_dialog_result = Details_button_delete then
	    database.remove_record_at_index( index )
	    populate_main_list
	 end
      end
   
   populate_main_list is
      local
	 record_iterator : INTEGER_16 
	 max_records : INTEGER_16
	 entry_texts : ARRAY[STRING]
	 current_line : STRING
      do
	 create current_line.make( 50 )
	 from
	    record_iterator := 0
	    max_records := database.number_of_records
	    create entry_texts.make( 0, max_records - 1 )
	    entry_texts.set_all_with( "woof" )
	 until
	    record_iterator >= max_records
	 loop
	    database.set_typed_record_from_index( record_iterator, current_record )
	    current_line.clear
	    current_line.append( current_record.name )
	    current_line.append( " (" )
	    current_line.append( current_record.quantity.to_string )
	    current_line.append( ":" )
	    current_line.add_last( character_from_alpha_code( current_record.alpha_code.to_integer ) )
	    current_line.append( ")" )
	    entry_texts.put( current_line.twin, record_iterator )
	    record_iterator := record_iterator + 1
	 end
	 form_manager.active_form.list_from_id( Main_form_list ).set_list_choices( entry_texts )
	 form_manager.active_form.list_from_id( Main_form_list ).draw
      end
   
   sort_database is
      do
	 sort_form.do_dialog
	 sort_comparison.set_sort_criteria( sort_form_event_handler.sort_first_by,
					    sort_form_event_handler.sort_second_by,
					    sort_form_event_handler.sort_third_by )
--	 form_manager.custom_alert( Debug_alert, sort_form_event_handler.sort_first_by.to_string,
--				    sort_form_event_handler.sort_second_by.to_string,
--				    sort_form_event_handler.sort_third_by.to_string )
	 database.quicksort( sort_comparison, 0 )
	 populate_main_list
      end
   
   sort_form : FORM
   
   sort_form_event_handler : SORT_FORM_EVENT_HANDLER
   
   details_form : FORM
   
   current_record : DEMO_RECORD
   
   database : RECORD_DATABASE
   
   last_handle : MEM_HANDLE
   
   sort_comparison : DEMO_RECORD_COMPARISON
   
end -- class MAIN_FORM_EVENT_HANDLER
