
class MAIN_FORM_EVENT_HANDLER

inherit
   PALM_CHARS
   FORM_MANAGER_CLIENT
   MQFQUIZ_CONSTANTS
   DATABASE_MANAGER_CLIENT
   INTEGER_32_HELPER
   STANDARD_EVENT_HANDLER
      redefine
         handle_frm_open_event,
	 handle_ctl_select_event,
	 handle_pop_select_event,
	 handle_scl_repeat_event,
	 handle_key_down_event
      end
	 
creation
   make

feature {ANY}
   
   Creator_id : INTEGER_32 is 
      once 
	 Result := integer_32_from_characters( 'e', 'm', 'q', 'f' ) 
      end
   
   Type_id : INTEGER_32 is 
      once 
	 Result := integer_32_from_characters( 'e', 'm', 'q', 'f' ) 
      end
   
   database_is_valid : BOOLEAN is
      do
	 Result := application_database /= Void and then application_database.opaque_pointer.is_not_null
      end
   
   populate_sample_database( database : RECORD_DATABASE ) is
      do
	 database.new_typed_record( 0, current_question )
      end
   
   create_sample_databases is
	 --creates a couple of simple one-question databases to test 
	 --population of the database selection listbox
      local
	 database : DATABASE
	 id : UINT32
      do
	 database_manager.create_database( 0, "Sample MQF 1", Creator_id, Type_id, False )
	 id := database_manager.find_database( 0, "Sample MQF 1" )
	 database := database_manager.open_database( 0, id, 3 )
	 make_sample_question
	 populate_sample_database( database )
	 database.close
	 database_manager.create_database( 0, "Sample MQF 2", Creator_id, Type_id, False )
	 id := database_manager.find_database( 0, "Sample MQF 2" )
	 database := database_manager.open_database( 0, id, 3 )
	 current_question.set_answer_a( "Woof")
	 populate_sample_database( database )
	 populate_sample_database( database )
	 database.close
      end
   
   make_sample_question is
      do
	 current_question.set_reference( "Welcome!" )
	 current_question.set_question( "Choose a quiz using the dropdown box above; navigate using the 'prev', 'random', and 'next' functions." )
	 current_question.set_answer_a( "" )
	 current_question.set_answer_b( "" )
	 current_question.set_answer_c( "" )	 
	 current_question.set_answer_d( "" )
	 current_question.set_answer_key( 1 )
      end
	 
   
   handle_frm_open_event( event : EVENT ) is
      do
--	 create_sample_databases
	 populate_database_list

	 form_manager.active_form.draw
	 make_sample_question
	 show_current_question_frame
         form_manager.alert( Splash_dialog )         
	 last_event_was_handled := True
      end
   
feature {ANY} -- data features
   
   application_database : RECORD_DATABASE
   
   question_indices : ARRAY[INTEGER_16]
   
   current_question_database_index : INTEGER_16 is
      require
	 question_indices /= Void
      do
	 Result := question_indices.item( question_counter )
      end
   
   question_counter : INTEGER_16
   
   mode : INTEGER_16
   
   Browse_mode : INTEGER_16 is unique
   
   Key_mode : INTEGER_16 is unique
   
   Quiz_mode : INTEGER_16 is unique
   
   current_question : QUIZ_QUESTION
   
   quiz_dialog : FORM
   
   set_indices_capacity( new_capacity : INTEGER_16 ) is
      do
	 if question_indices = Void then
	    create question_indices.with_capacity( new_capacity, 1 )
	 else
	    question_indices.clear
	    if question_indices.capacity < new_capacity then
	       question_indices.with_capacity( new_capacity, 1 )
	    end
	 end
      end
   
   set_indices_to_full is
      do
	 if database_is_valid then 
	    set_indices_to_identity( application_database.number_of_records )
	 else 
	    set_indices_to_identity( 1 )
	 end
      end
   
   set_indices_to_identity( number_of_records : INTEGER_16 ) is
      local
	 i : INTEGER_16
      do
	 set_indices_capacity( number_of_records )
	 from
	    i := 0
	 until
	    i = number_of_records
	 loop
	    question_indices.add_last( i )
	    i := i + 1
	 end
      end
   
   set_indices_to_bounds( lower, upper : INTEGER_16 ) is
      local
	 i, new_lower, new_upper : INTEGER_16
      do
	 new_lower := lower.min( upper )
	 new_upper := lower.max( upper )
	 new_lower := new_lower.max( 0 )
	 new_upper := new_upper.min( application_database.number_of_records - 1 )
	 set_indices_capacity( new_upper - new_lower + 1 )
	 from
	    i := new_lower
	 until
	    i > new_upper
	 loop
	    question_indices.add_last( i )
	    i := i + 1
	 end
      end
	 
   
   open_database( card_number : INTEGER_16; name : STRING ) is
      local
	 id : LOCAL_ID
      do
	 id := database_manager.find_database( card_number, name )
	 if database_is_valid then 
	    application_database.close
	 end
	 create application_database.from_database( database_manager.open_database( card_number, id, 3 ) )
	 if not database_is_valid then
	    form_manager.custom_alert( Debug_alert, "Unable to open database", 
				       "", "" )
	 else
	    set_indices_to_full
	    question_counter := question_indices.lower.to_integer_16
	    load_current_question_from_index( current_question_database_index )	    
	    set_browse_mode
	 end
      end
   
   make is
      do
--	 create database
	 mode := Browse_mode
	 create current_question
	 create quiz_dialog.from_form_id( Quiz_form )
      end
   
feature {ANY} -- event handlers
   
   handle_ctl_select_event( event : EVENT ) is
      do
	 inspect event.data_ctlselect_controlid
	 when Main_form_next_button then
	    on_next_question
	    last_event_was_handled := True
	 when Main_form_previous_button then
	    on_previous_question
	    last_event_was_handled := True
	 when Main_form_random_button then
	    on_random_question
	    last_event_was_handled := True
	 when 
	    Main_form_answer_a,
	    Main_form_answer_b,
	    Main_form_answer_c,
	    Main_form_answer_d
	  then
	    on_select_answer( answer_from_answer_control( event.data_ctlselect_controlid ) )
	    last_event_was_handled := True
	 when Main_form_edit_button then
	    on_edit
	    last_event_was_handled := True
	 else
	    last_event_was_handled := False
	 end
      end
   
   handle_key_down_event( event : EVENT ) is
      do
	 inspect event.data_keydown_chr
	 when Page_up_chr then
	    on_previous_question
	    last_event_was_handled := True
	 when Page_down_chr then
	    on_next_question
	    last_event_was_handled := True
	 else
	    last_event_was_handled := False
	 end
      end
	 
   
   set_question_counter( new_counter : INTEGER_16 ) is
      do
	 if question_indices /= Void then
	    question_counter := new_counter
	    if question_counter < question_indices.lower then
	       question_counter := question_indices.upper.to_integer_16
	    elseif question_counter > question_indices.upper then
	       question_counter := question_indices.lower.to_integer_16
	    end
	 end
	 
	 if database_is_valid then
	    load_current_question_from_index( current_question_database_index )
	    show_current_question_frame
	 end
      end
   
   on_next_question is
      do
	 set_question_counter( question_counter + 1 )
      end
   
   on_previous_question is
      do
	 set_question_counter( question_counter - 1 )
      end
   
   rng : MIN_STAND
   
   on_random_question is
      do
	 if rng = Void then create rng.make end
	 rng.next
	 if database_is_valid then
	    set_question_counter( rng.last_integer( question_indices.lower + question_indices.count ).to_integer_16 - 1 )
	 end
      end
   
   store_answer_as_current_question_key( answer : INTEGER_16 ) is
	 --set the current question's answer key to the given answer 
	 --and store it in the database
      do
	 current_question.set_answer_key( answer.to_integer_16 )
	 if application_database /= Void then
	    application_database.write_typed_record_to_index( current_question_database_index, current_question )
	 end
      end
   
   
   on_select_answer( answer : INTEGER_16 ) is
      do
	 inspect mode
	 when Browse_mode then
	    --don't do anything in browse mode--reset to the stored answer!
	    show_possible_answers
	 when Key_mode then
	    --in key mode, set the key for the answer and record it
	    store_answer_as_current_question_key( answer )
	 when Quiz_mode then
	    if answer = current_question.answer_key then
	       question_indices.remove( question_counter )
	       if question_indices.count = 0 then
		  if database_is_valid then
		     set_indices_to_full
		  end
		  form_manager.alert( Quiz_complete_alert )
		  set_browse_mode
	       else
		  set_question_counter( question_counter )
	       end
	    else
	       form_manager.custom_alert( Incorrect_answer_alert, current_correct_answer_text, "", "" )
	       on_next_question
	    end
	 end
	 
      end
      
   current_correct_answer_text_result : STRING
   
   current_correct_answer_text : STRING is
      do
	 if current_correct_answer_text_result = Void then
	    create current_correct_answer_text_result.make( 50 )
	 end
	 current_correct_answer_text_result.clear
	 Result := current_correct_answer_text_result
	 inspect current_question.answer_key
	 when 1 then
	    Result.append( "A. " )
	    Result.append( current_question.answer_a )
	 when 2 then
	    Result.append( "B. " )
	    Result.append( current_question.answer_b )
	 when 3 then
	    Result.append( "C. " )
	    Result.append( current_question.answer_c )
	 when 4 then
	    Result.append( "D. " )
	    Result.append( current_question.answer_d )
	 else
	    Result.append( "Answer key has not been set" )
	 end
      end
	    
      
   on_edit is
      do
      end
   
   handle_pop_select_event( event : EVENT ) is
      do
	 inspect event.data_popselect_controlid
	 when Main_form_database_popup then
	    on_select_database( event )
	    last_event_was_handled := True
	 when Main_form_mode_popup then
	    on_select_mode( event )
	    last_event_was_handled := True
	 else
	    last_event_was_handled := False
	 end
      end
   
   on_select_database( event : EVENT ) is
      do
	 open_database( 0, event.data_popselect_list.selection_text( event.data_popselect_selection ) )
      end
   
   set_mode_popup_text( text : STRING ) is
      do
	 form_manager.active_form.control_from_id( Main_form_mode_popup ).set_label( text ) 
      end
   
   
   set_mode( new_mode : INTEGER_16; mode_label : STRING ) is
      do
	 mode := new_mode
	 set_mode_popup_text( mode_label )
	 if database_is_valid then
	    load_current_question_from_index( current_question_database_index )
	 end
	 show_current_question_frame
      end
   
   set_browse_mode is
      do
	 if mode = Quiz_mode then
	    set_indices_to_full
	 end
	 set_mode( Browse_mode, "Browse" )
      end
   
   set_key_mode is
      do
	 if mode = Quiz_mode then
	    set_indices_to_full
	 end
	 set_mode( Key_mode, "Key" )
      end
   
   set_quiz_mode is
      do
	 set_mode( Quiz_mode, "Quiz" )
      end
   
   on_select_mode( event : EVENT ) is 
      do
	 inspect event.data_popselect_selection
	 when 0 then 
	    set_browse_mode
	 when 1 then 
	    set_key_mode
	 when 2 then 
	    if database_is_valid then
	       set_indices_to_full
	       if question_indices /= Void then
		  quiz_dialog.field_from_id( Quiz_form_from_field ).set_text_to_string( question_indices.lower.to_string )
		  quiz_dialog.field_from_id( Quiz_form_to_field ).set_text_to_string( question_indices.upper.to_string )
	       else
		  quiz_dialog.field_from_id( Quiz_form_from_field ).set_text_to_string ( "" )
		  quiz_dialog.field_from_id( Quiz_form_to_field ).set_text_to_string ( "" )	       
	       end
	       quiz_dialog.do_dialog
	       if quiz_dialog.last_dialog_result = Quiz_form_button_ok then
		  set_indices_to_bounds( quiz_dialog.field_from_id( Quiz_form_from_field ).contents.to_integer.to_integer_16 - 1,
					 quiz_dialog.field_from_id( Quiz_form_to_field ).contents.to_integer.to_integer_16 - 1 )
		  set_question_counter( question_indices.lower.to_integer_16 )
		  set_quiz_mode
	       end
	    end
	 end

      end
   
   handle_scl_repeat_event( event : EVENT ) is
      do
	 inspect event.data_sclrepeat_scrollbarid
	 when Main_form_question_scrollbar then
	    scroll_field( Main_form_question_field, Main_form_question_scrollbar, event.data_sclrepeat_newvalue - event.data_sclrepeat_value )
	    last_event_was_handled := False
	 when Main_form_answers_scrollbar then
	    scroll_field( Main_form_answers_field, Main_form_answers_scrollbar, event.data_sclrepeat_newvalue - event.data_sclrepeat_value )
	    last_event_was_handled := False
	 else
	    last_event_was_handled := False
	 end
      end
   
   
feature {ANY} -- show current question to form
   
   show_current_question_frame is
      do
	 show_question
	 show_answers
	 show_reference
	 show_possible_answers
	 show_counter
      end
   
   set_field_text( id : INTEGER_16; text : STRING ) is
      do
	 form_manager.active_form.field_from_id( id ).set_text_to_string( text )
	 form_manager.active_form.field_from_id( id ).draw
      end
   
   update_field_scrollbar( field_id, scrollbar_id : INTEGER_16 ) is
      do
	 form_manager.active_form.scrollbar_from_id( scrollbar_id ).set_scrollbar_values( form_manager.active_form.field_from_id( field_id ).scrollbar_values )
      end
   
   scroll_field( field_id, scrollbar_id, lines_to_scroll : INTEGER_16 ) is
      do
	 form_manager.active_form.field_from_id( field_id ).scroll( lines_to_scroll )
	 update_field_scrollbar( field_id, scrollbar_id )
      end
   
   
   show_correct_answer is
      do
      end
   
   show_question is
      do
	 if current_question.question.count > 510 then 
	    form_manager.custom_alert( Debug_alert, "Question too long", "", "" )
	 end
	 
	 set_field_text( Main_form_question_field, current_question.question )
	 update_field_scrollbar( Main_form_question_field, Main_form_question_scrollbar )
      end
   
   answer_text : STRING is once create Result.make(512) end
   
   --implemented as once variable to reduce number of dynamic allocations
   
   show_answers is
      do
	 answer_text.clear
	 answer_text.append( "A) " )
	 answer_text.append( current_question.answer_a )
	 answer_text.append( "%N" )
	 answer_text.append( "B) " )
	 answer_text.append( current_question.answer_b )
	 answer_text.append( "%N" )	 
	 answer_text.append( "C) " ) 
	 answer_text.append( current_question.answer_c ) 
	 answer_text.append( "%N" )	 
	 answer_text.append( "D) " ) 
	 answer_text.append( current_question.answer_d )	 
	 
	 if answer_text.count > 510 then 
	    form_manager.custom_alert( Debug_alert, "Answer too long", "", "" )
	 end

	 set_field_text( Main_form_answers_field, answer_text )
	 update_field_scrollbar( Main_form_answers_field, Main_form_answers_scrollbar )
      end
   
   show_reference is 
      do
	 
	 if current_question.reference.count > 119 then 
	    form_manager.custom_alert( Debug_alert, "Reference too long", "", "" )
	 end
	 
	 set_field_text( Main_form_reference_field, current_question.reference )
      end
   
   counter_text : STRING
   
   show_counter is
	 --show the question counter and number of questions in the 
	 --quiz.  This is done using counter_text as a data member 
	 --and append_in to save memory
      do
	 if counter_text = Void then create counter_text.make(8) end
	 counter_text.clear
	 question_counter.append_in( counter_text )
	 counter_text.add_last( '/' )
	 if question_indices = Void then 
	    counter_text.add_last( '?' )
	 else
	    question_indices.count.append_in( counter_text )
	 end
	 form_manager.active_form.field_from_id( Main_form_counter_field ).set_text_to_string( counter_text ) 
	 form_manager.active_form.field_from_id( Main_form_counter_field ).draw
      end
   
   show_possible_answers is
      do
	 inspect mode
	 when Browse_mode, Key_mode then
	    show_current_answer_key_as_selected_answer
	 else
	    clear_possible_answers
	 end
      end
   
   show_current_answer_key_as_selected_answer is
      do
	 if answer_control_from_answer( current_question.answer_key ) = 0 then
	    clear_possible_answers
	 else
	    form_manager.active_form.set_control_group_selection_by_id( 1, answer_control_from_answer( current_question.answer_key ) )
	 end
      end
   
   clear_possible_answers is
      do
	 form_manager.active_form.set_control_value_by_id( Main_form_answer_a, 0 )
	 form_manager.active_form.set_control_value_by_id( Main_form_answer_b, 0 )
	 form_manager.active_form.set_control_value_by_id( Main_form_answer_c, 0 )
	 form_manager.active_form.set_control_value_by_id( Main_form_answer_d, 0 )
      end
   
feature {ANY} -- conversion features
   
   answer_from_answer_control( control_id : INTEGER_16 ) : INTEGER_16 is
      do
	 inspect control_id
	 when Main_form_answer_a then Result := 1
	 when Main_form_answer_b then Result := 2
	 when Main_form_answer_c then Result := 3
	 when Main_form_answer_d then Result := 4
	 else Result := 0
	 end
      end
   
   answer_control_from_answer( answer : INTEGER_16 ) : INTEGER_16 is
      do
	 inspect answer
	 when 1 then Result := Main_form_answer_a
	 when 2 then Result := Main_form_answer_b
	 when 3 then Result := Main_form_answer_c
	 when 4 then Result := Main_form_answer_d
	 else Result := 0
	 end
      end
   
   
feature {ANY} -- database access   
   
   database_list_names : ARRAY[ STRING ]
   
   converted_list_choices : ARRAY[ POINTER ]
   
   populate_database_list is
	 --search for all databases with our creator and ID,
	 --then populate the list box with their names.
	 --we will use the strings as lookups later to load the
	 --databases
      local
	 state : DATABASE_MANAGER_SEARCH_STATE
      do
	 if database_list_names = Void then 
	    create database_list_names.make( 0, 10 )
	 end
	 create state.as_new_memory
	 database_list_names.clear
	 from
	    database_manager.get_next_database_by_type_and_creator( True, state, Type_id, Creator_id, False )
	 until
	    database_manager.last_error /= 0
	 loop
	    
	    database_manager.get_database_info( database_manager.last_next_database_card_number, database_manager.last_next_database_local_id )
	    database_list_names.add_last( database_manager.last_database_info_name.twin )
	    database_manager.get_next_database_by_type_and_creator( False, state, Type_id, Creator_id, False )
	 end
	 converted_list_choices := form_manager.active_form.list_from_id( Main_form_database_list ).pointer_array_from_string_array( database_list_names )
	 form_manager.active_form.list_from_id( Main_form_database_list ).set_list_choices_to_pointer_array( converted_list_choices )
      end
   
   open_as_current_database( card_number : INTEGER_16; local_id : UINT32 ) is
      do
      end
   
   load_current_question_from_index( index : INTEGER_16 ) is
      require 
	 application_database /= Void
	 current_question /= Void
      do
	 application_database.set_typed_record_from_index( index, current_question )
      end
   
   write_current_question_to_index( index : INTEGER_16 ) is
      do
      end
   

end -- class MAIN_FORM_EVENT_HANDLER
