class DEMO_TABLE
   
inherit
   TABLE_CONSTANTS
   TABLE_STYLES
   TABLE_DISPATCHER_CLIENT
   WINDOW_MANAGER_CLIENT
   FORM_MANAGER_CLIENT
   FORM_TABLE
      redefine
	 draw_item, load_data, save_data
      end
	 
creation	 
   from_shared_memory
	 
feature {ANY}
   
   draw_item( row, column : INTEGER_16; drawn_item_bounds: PALM_RECTANGLE ) is 
      local
	 x1, y1, x2, y2 : INTEGER_16
      do 
	 if column = 8 then
	    inspect item_integer( row, column )
	    when 0 then
	       x1 := drawn_item_bounds.left
	       y1 := drawn_item_bounds.top
	       x2 := drawn_item_bounds.right
	       y2 := drawn_item_bounds.bottom
	    when 1 then
	       x1 := drawn_item_bounds.left + ( drawn_item_bounds.width // 2 )
	       y1 := drawn_item_bounds.top
	       x2 := x1
	       y2 := drawn_item_bounds.bottom
	    when 2 then
	       x1 := drawn_item_bounds.right
	       y1 := drawn_item_bounds.top
	       x2 := drawn_item_bounds.left
	       y2 := drawn_item_bounds.bottom
	    when 3 then
	       x1 := drawn_item_bounds.left
	       y1 := drawn_item_bounds.top + ( drawn_item_bounds.height // 2 )
	       x2 := drawn_item_bounds.right
	       y2 := y1
	    else
	       x1 := drawn_item_bounds.left
	       y1 := drawn_item_bounds.top
	       x2 := x1
	       y2 := y1
	    end
	    window_manager.draw_line( x1, y1, x2, y2 )
	 end	 
      end

   
   handle_column_from_table_column( table_column : INTEGER_16) : INTEGER_16 is
      --return the handle column (1-3; eiffel arrays start at 1)
      do
	 inspect 
	    table_column
	 when 0 then
	    Result := 1
	 when 5 then
	    Result := 2
	 when 7 then
	    Result := 3
	 end
      end
   
   handle : MEM_HANDLE
   
   load_data( row, column : INTEGER_16; is_editable : BOOLEAN; field : FORM_FIELD ) is 
      do 
	 if handle = Void then 
	    create handle.allocate( 5, True )
	    handle.set_to_string("bob")
	 end
	 last_load_data_was_successful := True
	 last_load_data_handle := handles.item( handle_column_from_table_column( column )).item(row + 1)
	 --add 1 to row because eiffel arrays start at 1
	 last_load_data_size := last_load_data_handle.size.to_integer_16
	 last_load_data_offset := 0
      end
      
      
      
   save_data( row, column : INTEGER_16 ) is 
      local
	 handle_column : INTEGER_16
	 field : FORM_FIELD
	 s : STRING
      do 
	 table_should_be_redrawn_after_save := False
	 handle_column := handle_column_from_table_column( column )
	 if ( row = 0 and handle_column = 2 ) then
	    field := current_field
	    if field.opaque_pointer.is_not_null and then field.is_dirty then
	       s := handles.item( handle_column ).item( row+1 ).to_string
	       s.to_lower
	       handles.item(handle_column).item(row+1).set_to_string( s )
	       mark_row_invalid( row )
	       table_should_be_redrawn_after_save := True
	       form_manager.active_form.field_from_id( Main_form_field ).set_text_to_string( s )
	       form_manager.active_form.field_from_id( Main_form_field ).draw
	    end
	 end
      end
   
feature {ANY}   -- initialization
   
   handles : ARRAY[ ARRAY[ MEM_HANDLE ] ]
	 --holds text handles used in the table
   
   allocate_handles is
      local
	 i, j : INTEGER_16
	 h : MEM_HANDLE
	 new_array : ARRAY[ MEM_HANDLE ]
      do
	 create handles.make( 1, 3 )
	 from i := 1 until i = 4 loop
	    create new_array.make( 1, 12 )
	    handles.put( new_array, i )
	    from j := 1 until j = 12 loop
	       create h.allocate( 2, True )
	       handles.item( i ).put( h, j )
	       handles.item(i).item(j).set_to_string( "" )
	       j := j + 1
	    end
	    i := i + 1
	 end
      end
	 
   
   initialize( form : FORM ) is
      local
	 row : INTEGER_16
	 column : INTEGER_16
	 num_rows : INTEGER_16
	 labels : ARRAY[STRING]
	 list : FORM_LIST
      do
	 if handles = Void then allocate_handles end
	 labels := << "0", "1", "2", "3", "4", "5", "6", "7" >>
	 --set column widths.  For no apparent reason, this is required.
	 --it should not be
	 set_column_width( 0, 25 )
	 set_column_width( 1, 6 )
	 set_column_width( 2, 12 )
	 set_column_width( 3, 6 )
	 set_column_width( 4, 25 )
	 set_column_width( 5, 25 )
	 set_column_width( 6, 19 )
	 set_column_width( 7, 25 )
	 set_column_width( 8, 15 )
	 
	 list := form.list_from_id( Main_form_list )
	 
	 num_rows := number_of_rows
 	 
	 from
	    row := 0
	 until
	    row = num_rows
	 loop
	    set_item_style( row, 0, Text_with_note_table_item )
	    
	    set_item_style( row, 1, Numeric_table_item )
	    set_item_integer( row, 1, row )
	    
	    set_item_style( row, 2, Checkbox_table_item )
	    set_item_integer( row, 2, row \\ 2 )
	    
	    set_item_style( row, 3, Label_table_item )
	    set_item_pointer( row, 3, labels.item( row+1 ).to_external )
	    --use "row+1" above because eiffel arrays start with item 1!
	    
	    set_item_style( row, 4, Date_table_item )
	    set_item_integer( row, 4, 0 )
	    
	    set_item_style( row, 5, Text_table_item )
	    
	    set_item_style( row, 6, Popup_trigger_table_item )
	    set_item_integer( row, 6, row \\ 5 )
	    set_item_pointer( row, 6, list.opaque_pointer )
	    
	    set_item_style( row, 7, Narrow_text_table_item )
	    set_item_integer( row, 7, row*2 )
	    
	    set_item_style( row, 8, Custom_table_item )
	    set_item_integer( row, 8, row \\ 4 )
	    
	    set_row_is_usable( row, True )
	    set_row_is_selectable( row, True )
	    set_row_height( row, 11 )
	    mark_row_invalid( row )
	 
	    row := row + 1
	 end
	 
	 set_row_is_usable( 1, False )
	 
	 from
	    column := 0
	 until
	    column = 9
	 loop
	    set_column_is_usable( column, True )
	    column := column + 1
	 end
	 
	 table_dispatcher.register_table_with_custom_load_data_procedure( Current, 0 )
	 table_dispatcher.register_table_with_custom_load_data_procedure( Current, 5 )	 
	 table_dispatcher.register_table_with_custom_load_data_procedure( Current, 7 )	 
	 table_dispatcher.register_table_with_custom_draw_event( Current, 7 )
	 table_dispatcher.register_table_with_custom_draw_event( Current, 8 )
	 table_dispatcher.register_table_with_custom_save_data_procedure( Current, 5 )
      end   
end

   
   
