/*
-- This file is  free  software, which  comes  along  with  SmartEiffel. This
-- software  is  distributed  in the hope that it will be useful, but WITHOUT
-- ANY  WARRANTY;  without  even  the  implied warranty of MERCHANTABILITY or
-- FITNESS  FOR A PARTICULAR PURPOSE. You can modify it as you want, provided
-- this header is kept unaltered, and a notification of the changes is added.
-- You  are  allowed  to  redistribute  it and sell it, alone or as a part of
-- another product.
--       Copyright (C) 1994-2002 LORIA - INRIA - U.H.P. Nancy 1 - FRANCE
--          Dominique COLNET and Suzanne COLLIN - SmartEiffel@loria.fr
--                       http://SmartEiffel.loria.fr
--
*/
/*
  This file (SmartEiffel/sys/runtime/base.c) is included for _all_ modes of
  compilation (-boost, -no_check, ... -all_check, -debug_check).
*/

/*
  Byte swapping function
*/
void copy_swap_16(const uint16_t *src, uint16_t *dest, int count){
  while (count--) {
    *dest++ = (*src << 8) | (*src >> 8);
    src++;
  }
}


/*
  The wrapper for `malloc' (generated C code is supposed to use
  only `se_malloc' instead of direct `malloc').
*/
void* se_malloc(size_t size) {
  void *result = malloc(size);
  if (result == NULL) {
#ifdef SE_EXCEPTIONS
    internal_exception_handler(No_more_memory);
#elif !defined(SE_BOOST)
    error0("No more memory.", NULL);
#else
  fprintf(SE_ERR,"No more memory.\n");
  se_print_run_time_stack();
  exit(EXIT_FAILURE);
#endif
  }
  return result;
}

/*
  The wrapper for `calloc' (generated C code is supposed to use
  only `se_calloc' instead of direct `calloc').
*/
void* se_calloc(size_t nmemb, size_t size) {
  void *result = calloc(nmemb,size);
  if (result == NULL) {
#ifdef SE_EXCEPTIONS
    internal_exception_handler(No_more_memory);
#elif !defined(SE_BOOST)
    error0("No more memory.", NULL);
#else
  fprintf(SE_ERR,"No more memory.\n");
  se_print_run_time_stack();
  exit(EXIT_FAILURE);
#endif
  }
  return result;
}

/*
  The wrapper for `realloc' (generated C code is supposed to use
  only `se_realloc' instead of direct `realloc').
*/
void* se_realloc(void* src, size_t size) {
  void *result = realloc(src, size);
  if (result == NULL) {
#ifdef SE_EXCEPTIONS
    internal_exception_handler(No_more_memory);
#elif !defined(SE_BOOST)
    error0("No more memory.", NULL);
#else
  fprintf(SE_ERR,"No more memory.\n");
  se_print_run_time_stack();
  exit(EXIT_FAILURE);
#endif
  }
  return result;
}

/*
  Section added by VPutz for important items such as PilotMain which are
  absolutely necessary in epalm!
*/

int main( int argc, char*argv[]);

UInt32
PilotMain (UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags) {
  int Result = 0;
  if (cmd == sysAppLaunchCmdNormalLaunch) {
    Result = main( 0, NULL );
  }
  return Result;
}

/* vputz in PalmOS unix_stdio.h, stderr is defined as an unsigned char* */
int fprintf( unsigned char* p, const char* format, ... )
{
  /*do nothing*/
  return 0;
}

int printf( const char* format, ... )
{
  return 0;
}

/* vputz -- this dispatches events to the form_dispatcher once object */
/*   defined in the FORM_DISPATCHER_CLIENT class */
Boolean form_dispatcher_event_handler( EventType* eventp )
{
  Boolean handled = false;
#ifdef __GNUC__
CALLBACK_PROLOGUE
#endif
  handled = (Boolean)(FORM_DISPATCHER_dispatch_event_from_pointer( FORM_DISPATCHER_CLIENT_form_dispatcher(eiffel_root_object), eventp ));
#ifdef __GNUC__
CALLBACK_EPILOGUE
#endif
  return handled;
}

/* vputz -- some rather criminal things going on here.  SmartEiffel does a */
/* lot of exit calls, but PalmOS doesn't really know how to handle this. */
/* the exit() function calls the on_exit handlers, but then calls */
/*  extern void _cleanup( void ) and */
/*  extern void _exit( int ) __attribute__ ((__noreturn__)) */
/* (see libgcc2.c, I think).  So we define them here; _cleanup does nothing, */
/* and _exit just exits. */
/* addendum-- we're just going to redefine exit in the base.h file and see what happens */

void epalm_exit( int code )
{
}

void _cleanup( void )
{
}

void _exit( int i )
{
}
