indexing
   contents: "Table dispatcher object--forwards calls to custom draw/load/save"
   author : "Victor B. Putz"
   completed : no
   tested : no
   
class TABLE_DISPATCHER
   --forwards calls to custom save, load, and draw events to appropriate
   --registered table objects
   
creation
   make
   
feature {NONE}
   
   draw_dispatch_map : DICTIONARY[ FORM_TABLE, POINTER ]
   load_data_dispatch_map : DICTIONARY[ FORM_TABLE, POINTER ]
   save_data_dispatch_map : DICTIONARY[ FORM_TABLE, POINTER ]
   
   make is
      do
	 create draw_dispatch_map.with_capacity(3)
	 create load_data_dispatch_map.with_capacity(3)
	 create save_data_dispatch_map.with_capacity(3)
      end
   
   
   dispatch_custom_draw_event( p_table : POINTER; row, column : INTEGER_16; p_bounds : POINTER ) is
	 --dispatches a "custom draw" event--this is called from C 
	 --helper code and should NOT be called  from application code
      local
	 bounds : PALM_RECTANGLE
      do
	 if draw_dispatch_map.has( p_table ) then
	    create bounds.from_shared_memory( p_bounds )
	    draw_dispatch_map.at( p_table ).draw_item( row, column, bounds )
	 end
      end
   
   dispatch_custom_load_data_event( p_table : POINTER; 
				    row, column : INTEGER_16; 
				    is_editable : BOOLEAN; 
				    p_handle, p_offset, p_size, p_field, p_load_was_successful : POINTER ) is
	 --dispatches a "custom load data" event--this is called from C 
	 --helper code and should NOT be called from application code
      local
	 table : FORM_TABLE
	 field : FORM_FIELD
      do
	 if load_data_dispatch_map.has( p_table ) then
	    create field.from_shared_memory( p_field )
	    table := load_data_dispatch_map.at( p_table )
	    table.load_data( row, column, is_editable, field )
	    table_dispatcher_store_load_data_return_values( table.last_load_data_handle.handle, 
							    p_handle, 
							    table.last_load_data_offset.to_integer_16, 
							    p_offset, 
							    table.last_load_data_size.to_integer_16, 
							    p_size, 
							    table.last_load_data_was_successful, 
							    p_load_was_successful )
	 end
      end
   
   dispatch_custom_save_data_event( p_table : POINTER; 
				    row, column : INTEGER_16 ) : BOOLEAN is
	 --dispatches a "custom save data" event--this is called from 
	 --C helper code and should NOT be called from application code
      do
	 if save_data_dispatch_map.has( p_table ) then
	    save_data_dispatch_map.at( p_table ).save_data( row, column )
	    Result := save_data_dispatch_map.at( p_table ).table_should_be_redrawn_after_save
	 else
	    Result := false
	 end
      end
   
feature {ANY}   
   
   register_table_with_custom_draw_event( table : FORM_TABLE; column : INTEGER_16 ) is
	 --register a table to have a custom draw procedure for the 
	 --given column
      do
	 table_dispatcher_register_table_with_custom_draw_procedure( table.opaque_pointer, column.to_integer_16 )
	 draw_dispatch_map.put( table, table.opaque_pointer )
      end
   
   register_table_with_custom_load_data_procedure( table : FORM_TABLE; column : INTEGER_16 ) is
	 --register a table to have a custom load data procedure for 
	 --the given column
      do
	 table_dispatcher_register_table_with_custom_load_data_procedure( table.opaque_pointer, column.to_integer_16 )
	 load_data_dispatch_map.put( table, table.opaque_pointer )
      end
   
   register_table_with_custom_save_data_procedure( table : FORM_TABLE; column : INTEGER_16 ) is
	 --register a table to have a custom save data procedure for 
	 --the given column
      do
	 table_dispatcher_register_table_with_custom_save_data_procedure( table.opaque_pointer, column.to_integer_16 )
	 save_data_dispatch_map.put(table, table.opaque_pointer)
      end
   
feature {NONE} -- C helpers   
   
   table_dispatcher_store_load_data_return_values( h, p_h : POINTER; 
						   offset : INTEGER_16; 
						   p_offset : POINTER; 
						   size : INTEGER_16; 
						   p_size : POINTER; 
						   last_load_successful : BOOLEAN; 
						   p_last_load_successful : POINTER ) is external "C" end
   
   table_dispatcher_register_table_with_custom_draw_procedure( p_table : POINTER; column : INTEGER_16 ) is external "C" end
   
   table_dispatcher_register_table_with_custom_load_data_procedure( p_table : POINTER; column : INTEGER_16 ) is external "C" end
   
   table_dispatcher_register_table_with_custom_save_data_procedure( p_table : POINTER; column : INTEGER_16 ) is external "C" end
   
end

   
