
indexing
   contents: "A singleton-style database with a given type and Id, which can open and populate itself.  Intended for applications which maintain a single persistent data store and reference it every run"
   Author : "Victor B. Putz"
   completed : no
   tested : no
	     
class APPLICATION_DATABASE
   
inherit
   RECORD_DATABASE; INTEGER_32_HELPER
   
creation
   from_database, from_type_and_creator, from_type_and_creator_using_characters
   
feature {ANY}
   
   from_type_and_creator( type_id, creator_id : INTEGER_32; name : STRING; mode : INTEGER_16 ) is
	 --something of a kitchen-sink function, this opens the 
	 --database with the given type and creator ID if one exists. 
	 --If one does not exist, it is created using the given type 
	 --ID, creator ID, and name, and then populated using the 
	 --deferred "populate" feature.
      local
	 database : DATABASE
      do
	 database := database_manager.open_database_by_type_and_creator( type_id, creator_id, mode )
	 if database.is_not_null then
	    --if we've opened the database, just create this database
	    from_database( database )
	 else
	    --otherwise, try creating the database
	    database_manager.create_database( 0, name, creator_id, type_id, False )
	    if database_manager.last_error = 0 then
	       --okay, we've created the database; now open it
	       database := database_manager.open_database_by_type_and_creator( type_id, creator_id, mode )
	       if database.is_not_null then
		  --if database opens, create and populate it
		  from_database( database )
		  populate
	       end
	    end
	 end
      end
   
   from_type_and_creator_using_characters( type_a, type_b, type_c, type_d,
					   creator_a, creator_b,
					   creator_c, creator_d : CHARACTER; 
					   name : STRING;
					   mode : INTEGER_16 ) is
	 --same as from_type_and_creator, but using characters for 
	 --the IDs so one doesn't have to create temp variables in 
	 --client code for the UINT32s
      local
	 type_id, creator_id : INTEGER_32
      do
	 type_id := integer_32_from_characters( type_a, 
						type_b, 
						type_c, 
						type_d )
	 creator_id := integer_32_from_characters( creator_a,
						   creator_b,
						   creator_c,
						   creator_d )
	 from_type_and_creator( type_id, creator_id, name, mode )
      end
   
   populate is do end
   
end
	 
