indexing
   contents: "C structure creation/deletion and access"
   author: "Victor Putz, vputz@nyx.net, from ideas in C_STRUCTURE, elj-win32"
   completed: no
   tested: no

class C_STRUCT
   
inherit
   MEMORY
      redefine
	 dispose
      end
   
feature {ANY} -- creation
   
   from_shared_memory( p : POINTER ) is
	 --create with already-allocated memory pointed to by p; when
	 --collected, do not free the memory
      do
	 is_shared := True
	 opaque_pointer := p
      ensure
	 is_shared
	 opaque_pointer = p
      end
   
   from_orphaned_memory( p : POINTER ) is
	 --create with already-allocated memory pointed to by p; when
	 --collected, free the memory
      do
	 is_shared := False
	 opaque_pointer := p
      ensure
	 not is_shared
	 opaque_pointer = p
      end
   
   as_new_memory is
	 --create, allocating memory as a new object
	 --when collected, free the memory
      do
	 is_shared := False
	 opaque_pointer := c_struct_allocate_memory( structure_size )
	 c_struct_prepare_memory_as_new_object( opaque_pointer );
      ensure
	 not is_shared
	 opaque_pointer /= Void
      end
   
   opaque_pointer : POINTER
	 --pointer to the allocated memory
   
   is_null : BOOLEAN is
      do
	 Result := opaque_pointer = c_null_pointer
      end
   
   is_not_null : BOOLEAN is
      do
	 Result := not is_null
      end
   
feature {NONE} -- misc memory management
   
   c_struct_allocate_memory( size : INTEGER_32 ) : POINTER is
	 --allocate memory for a C structure, and fill it with zeros
      require
	 size >= 0
      external "C use <MemoryMgr.h>"
		alias "MemPtrNew"
      end
   
   c_struct_free( pointer : POINTER ) : INTEGER_16 is
	 --free memory from a C structure
      external "C use <MemoryMgr.h>"
		alias "MemPtrFree"
      end
   
   c_struct_prepare_memory_as_new_object( pointer : POINTER ) is
	 --redefine to do further memory setup
      do
      end
   
   dispose is
			--when collected, free memory if made as shared object
		local
			last_error : INTEGER_16
      do
			if ( not is_shared ) then
				if opaque_pointer.is_not_null then
					last_error := c_struct_free( opaque_pointer )
					opaque_pointer := c_null_pointer
				end
			end
      end
   
   is_shared : BOOLEAN
	 --whether or not the memory pointed to by opaque_pointer is shared
   
   structure_size : INTEGER_32 is deferred end
	 --size of the C structure
   
   c_null_pointer : POINTER is 
      external "C inline" 
		alias "NULL"
      end
   
end


