indexing
   contents : "wraps form functionality"
   author : "Victor B. Putz"
   completed : no
   tested : no
   
class FORM

inherit
   FORM_DISPATCHER_CLIENT
   C_STRUCT
      redefine
	 structure_size
      end
   
creation
   from_form_id, from_shared_memory, from_orphaned_memory, as_new_memory
   
feature {ANY}
   
   from_form_id( new_form_id : INTEGER_16 ) is
      do
	 from_shared_memory( c_form_init_form( new_form_id ) )
      end
   
   set_as_active_form is
      do
	 c_form_set_as_active_form( opaque_pointer )
      end
   
   set_event_handler( event_handler : EVENT_HANDLER ) is
      do
	 form_dispatcher.set_event_handler( event_handler, Current )
      end
   
   last_dialog_result : INTEGER_16
   
   do_dialog is
	 --shows the form as a dialog.  On exit, the last_dialog_result 
	 --feature contains the ID of the first button the user pushed
	 --(which closes the dialog)
      do
	 last_dialog_result := c_form_do_dialog( opaque_pointer )
      end
   
   draw is
      do
	 c_form_draw( opaque_pointer )
      end
   
   structure_size : INTEGER_32 is
      do
	 Result := c_form_structure_size
      end
   
feature {ANY} -- form object access
   
   label( id : INTEGER_16 ) : STRING is
      do
	 create Result.from_external_copy( c_form_get_label( opaque_pointer, id ) )
      end
   
   set_label( id : INTEGER_16; new_label : STRING ) is
      do
	 c_form_copy_label( opaque_pointer, id, new_label.to_external )
      end
   
   object_id_from_index( object_index : INTEGER_16 ) : INTEGER_16 is
      do
	 Result := c_form_get_object_id_from_index( opaque_pointer, object_index )
      end
   
   object_index_from_id( form_object_id : INTEGER_16 ) : INTEGER_16 is
      do
	 Result := c_form_get_object_index_from_id( opaque_pointer, form_object_id )
      end
   
   object_from_id( id : INTEGER_16 ) : FORM_OBJECT is
      do
	 create Result.from_shared_memory( c_form_get_object_pointer( opaque_pointer, object_index_from_id( id ) ) )
      end
   
   object_from_index( object_index : INTEGER_16 ) : FORM_OBJECT is
      do
	 create Result.from_shared_memory( c_form_get_object_pointer( opaque_pointer, object_index ) );
      end

   object_type_from_index( object_index : INTEGER_16 ) : INTEGER_16 is
      do
         Result := c_form_get_object_type( opaque_pointer, object_index )
      end  

   number_of_objects : INTEGER_16 is
      do
         Result := c_form_get_number_of_objects( opaque_pointer )
      end
   
   hide_object( object_index : INTEGER_16 ) is
      do
	 c_form_hide_object( opaque_pointer, object_index )
      end
   
   show_object( object_index : INTEGER_16 ) is
      do
	 c_form_show_object( opaque_pointer, object_index )
      end
   
feature {ANY} -- control group features
   
   control_group_selection( group_number : INTEGER_8 ) : INTEGER_16 is
	 --get the form index (NOT object ID) of a given control group
      do
	 Result := c_form_control_group_selection( opaque_pointer, group_number )
      end
   
   set_control_group_selection_by_id( group_number : INTEGER_8; 
												  form_object_id : INTEGER_16 ) is
	 --set the selected control in a group based on the object id, 
	 --NOT the form index (see disparity with control_group_selection)
      do
	 c_form_set_control_group_selection( opaque_pointer, group_number, form_object_id )
      end
   
feature {ANY} -- control value set/get
   
   set_control_value_by_index( object_index, new_value : INTEGER_16 ) is
      do
	 c_form_set_control_value( opaque_pointer, object_index, new_value )
      end
   
   set_control_value_by_id( id, new_value : INTEGER_16 ) is
      do
	 c_form_set_control_value( opaque_pointer, object_index_from_id( id ), new_value )
      end
   
   control_value_by_index( object_index : INTEGER_16 ) : INTEGER_16 is
      do
	 Result := c_form_get_control_value( opaque_pointer, object_id_from_index ( object_index ) )
      end
   
   control_value_by_id( id : INTEGER_16 ) : INTEGER_16 is
      do
	 Result := c_form_get_control_value( opaque_pointer, id )
      end
   
   c_form_get_control_value( p_form : POINTER; id : INTEGER_16 ) : INTEGER_16 is
      external "C use <Form.h>"
      alias "FrmGetControlValue"
      end
   
   c_form_set_control_value( p_form : POINTER; index, new_value : INTEGER_16 ) is
      external "C use <Form.h>"
      alias "FrmSetControlValue"
      end
   
feature {ANY} -- more typesafe form object access
   
   control_from_id( id : INTEGER_16 ) : FORM_CONTROL is
      local
	 index : INTEGER_16 
      do
	 index := object_index_from_id( id )
	 check 
	    object_type_from_index( index ) = Control_object_kind
	 end
	 create Result.from_shared_memory( c_form_get_object_pointer( opaque_pointer, index ) )
      end
   
   field_from_id( id : INTEGER_16 ) : FORM_FIELD is
      local
	 index : INTEGER_16
      do
	 index := object_index_from_id( id )
	 check
	    object_type_from_index( index ) = Field_object_kind
	 end
	 create Result.from_shared_memory( c_form_get_object_pointer( opaque_pointer, index ) )
      end
   
   list_from_id( id : INTEGER_16 ) : FORM_LIST is
      require
	 object_type_from_index( object_index_from_id( id ) ) = List_object_kind
      do
	 create Result.from_shared_memory( c_form_get_object_pointer( opaque_pointer, object_index_from_id( id ) ) )
      end
   
   scrollbar_from_id( id : INTEGER_16 ) : FORM_SCROLLBAR is
      require
	 object_type_from_index( object_index_from_id( id ) ) = Scrollbar_object_kind
      do
	 create Result.from_shared_memory( c_form_get_object_pointer( opaque_pointer, object_index_from_id( id )))
      end
   
   table_from_id( id : INTEGER_16 ) : FORM_TABLE is
      require
	 object_type_from_index( object_index_from_id( id ) ) = Table_object_kind
      do
	 create Result.from_shared_memory( c_form_get_object_pointer( opaque_pointer, object_index_from_id( id )))
      end
   
feature {ANY} -- form object types
   
   Field_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmFieldObj"
      end
   
   Control_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmControlObj"
      end
   
   List_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmListObj"
      end
   
   Table_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmTableObj"
      end
   
   Bitmap_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmBitmapObj"
      end
   
   Line_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmLineObj"
      end
   
   Frame_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmFrameObj"
      end
   
   Rectangle_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmRectangleObj"
      end
   
   Label_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmLabelObj"
      end
   
   Title_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmTitleObj"
      end
   
   Popup_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmPopupObj"
      end
   
   Graffiti_state_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmGraffitiStateObj"
      end
   
   Gadget_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmGadgetObj"
      end
   
   Scrollbar_object_kind : INTEGER_16 is
      external "C inline use <Form.h>"
      alias "frmScrollbarObj"
      end
   
feature {ANY} -- setting form control contents without accessing the controls (for LoD)
   
   set_field_contents( field_id : INTEGER_16; s : STRING ) is
      do
	 field_from_id( field_id ).set_text_to_string( s )
      end
   
   set_control_label( control_id : INTEGER_16; s : STRING ) is
      do
	 control_from_id( control_id ).set_label( s )
      end
   
   draw_field( field_id : INTEGER_16 ) is
      do
	 field_from_id( field_id ).draw
      end
   
   draw_control( control_id : INTEGER_16 ) is
      do
	 control_from_id( control_id ).draw
      end
   
feature {NONE}
   
   c_form_init_form( new_form_id : INTEGER_16 ) : POINTER is
      external "C use <Form.h>"
      alias "FrmInitForm"
      end
   
   c_form_set_as_active_form( pointer : POINTER ) is
      external "C use <Form.h>"
      alias "FrmSetActiveForm"
      end
   
   c_form_structure_size : INTEGER_32 is
      external "C inline use <Form.h>"
      alias "(sizeof (FormType))"
      end
   
   c_form_do_dialog( p : POINTER ) : INTEGER_16 is
      external "C use <Form.h>"
      alias "FrmDoDialog"
      end
   
   c_form_draw( pointer : POINTER ) is
      external "C use <Form.h>"
      alias "FrmDrawForm"
      end
   
   c_form_get_object_id_from_index( pointer : POINTER; object_index : INTEGER_16 ) : INTEGER_16 is
      external "C use <Form.h>"
      alias "FrmGetObjectId"
      end
   
   c_form_get_object_index_from_id( pointer : POINTER; form_object_id : INTEGER_16 ) : INTEGER_16 is
      external "C use <Form.h>"
      alias "FrmGetObjectIndex"
      end
   
   c_form_get_object_pointer( pointer : POINTER; object_index : INTEGER_16 ) : POINTER is
      external "C use <Form.h>"
      alias "FrmGetObjectPtr"
      end

   c_form_get_number_of_objects( pointer : POINTER ) : INTEGER_16 is
      external "C use <Form.h>"
      alias "FrmGetNumberOfObjects"
      end

   c_form_get_object_type( pointer : POINTER; object_index : INTEGER_16 ) : INTEGER_16 is
      external "C use <Form.h>"
      alias "FrmGetObjectType"
      end
   
   c_form_get_label( p : POINTER; label_index : INTEGER_16 ) : POINTER is
      external "C use <Form.h>"
	 alias "FrmGetLabel"
      end
   
   c_form_copy_label( p : POINTER; id : INTEGER_16; p_label : POINTER ) is
      external "C use <Form.h>"
      alias "FrmCopyLabel"
      end
   
   c_form_hide_object( p : POINTER; index : INTEGER_16 ) is
      external "C use <Form.h>"
      alias "FrmHideObject"
      end
   
   c_form_show_object( p : POINTER; index : INTEGER_16 ) is
      external "C use <Form.h>"
      alias "FrmShowObject"
      end
   
   c_form_control_group_selection( p : POINTER; 
				   group : INTEGER_8 ) : INTEGER_16 is
      external "C use <Form.h>"
      alias "FrmGetControlGroupSelection"
      end
   
   c_form_set_control_group_selection( p : POINTER; 
				       group : INTEGER_8; 
				       id : INTEGER_16 ) is
      external "C use <Form.h>"
      alias "FrmSetControlGroupSelection"
      end
   
end

	 
  
      
   
   
