indexing
   contents: "Field access for fill-in fields"
   author: "Victor Putz, vputz@nyx.net"
   completed: no
   tested: no

class FORM_FIELD
   
inherit
   FORM_OBJECT
      redefine
	 structure_size
      end
   EVENT_HANDLER
      redefine
	 handle_event
      end
   
creation
   from_shared_memory, from_orphaned_memory, as_new_memory
   
feature {ANY}
   
   structure_size : INTEGER_32 is
      do
	 Result := c_form_field_structure_size
      end
	 
   compact_text is
      do
	 c_form_field_compact_text( opaque_pointer )
      end
	 
   copy_selection_to_clipboard is
      do
	 c_form_field_copy_selection_to_clipboard( opaque_pointer )
      end
	 
   cut_selection_to_clipboard is
      do
	 c_form_field_cut_selection_to_clipboard( opaque_pointer )
      end
	 
   delete_range( sel_start, sel_end : INTEGER_16 ) is
      do
	 c_form_field_delete_selection( opaque_pointer, sel_start, sel_end )
      end
	 
   is_dirty : BOOLEAN is
      do
	 Result := c_form_field_is_dirty( opaque_pointer )
      end
	 
   draw is
      do
	 c_form_field_draw_field( opaque_pointer )
      end
	 
   erase is
      do
	 c_form_field_erase_field( opaque_pointer )
      end
	 
   free_memory is
	 --release the handle-based memory allocated to the field's 
	 --text and associated word-wrapping information
      do
	 c_form_field_free_memory( opaque_pointer )
      end
	 
   font_id : INTEGER_16 is
      do
	 Result := c_form_field_get_font( opaque_pointer )
      end
   
   insertion_point_position : INTEGER_16 is
      do
	 Result := c_form_field_get_insertion_point_position( opaque_pointer )
      end
	 
   max_chars : INTEGER_16 is
	 --max number of bytes the field can accept
      do
	 Result := c_form_field_get_max_chars( opaque_pointer )
      end
	 
   number_of_blank_lines : INTEGER_16 is
      do
	 Result := c_form_field_get_number_of_blank_lines( opaque_pointer )
      end
	 
   scroll_position : INTEGER_16 is
	 --offset of the first character in the first visible line of 
	 --the field.  This is, surprisingly enough, NOT related to 
	 --scrollbars in any way.  See the scrollbar_values function 
	 --for this if you need it.
      do
	 Result := c_form_field_get_scroll_position( opaque_pointer )
      end
   
   last_scroll_position_first_line : INTEGER_16
   last_scroll_position_text_height : INTEGER_16
   last_scroll_position_field_height : INTEGER_16
   
   get_scroll_values is
      -- update the last_scroll_position_* features with current values
      do
	 c_form_field_get_scroll_values( opaque_pointer,
					 $last_scroll_position_first_line,
					 $last_scroll_position_text_height,
					 $last_scroll_position_field_height )
      end
   
   scroll_position_first_line : INTEGER_16 is
	 --line of text that is the topmost visible line.  Line 
	 --numbering starts with 0
      do
	 get_scroll_values
	 Result := last_scroll_position_first_line;
	 -- Result := c_form_field_get_scroll_values( opaque_pointer, 0 )
      end
	 
   scroll_position_text_height : INTEGER_16 is
	 --total number of lines neebded to display the field's text given 
	 --the current width of the field
      do
	 get_scroll_values
	 Result := last_scroll_position_text_height
	 -- Result := c_form_field_get_scroll_values( opaque_pointer, 1 )
      end
	 
   scroll_position_field_height : INTEGER_16 is
	 --current number of visible lines in the field
      do
	 get_scroll_values
	 Result := last_scroll_position_field_height
	 -- Result := c_form_field_get_scroll_values( opaque_pointer, 2 )
      end
   
   scrollbar_values : SCROLLBAR_VALUES is
	 -- scrollbar values for the current field status and 
	 -- insertion point
      local
	 current_position : INTEGER_16
	 current_text_height : INTEGER_16
	 field_height : INTEGER_16
	 max_value : INTEGER_16
      do
	 current_position := scroll_position_first_line
	 current_text_height := scroll_position_text_height
	 field_height := scroll_position_field_height
	 if current_text_height > field_height then
	    max_value := current_text_height - field_height
	 elseif current_position /= 0 then
	    max_value := current_position;
	 else 
	    max_value := 0
	 end
	 
	 create Result
	 Result.set_values( current_position, 0, max_value, field_height - 1 )
      end
   
   last_selection_start : INTEGER_16
   last_selection_end : INTEGER_16
   
   get_selection is
      do
	 last_error := c_form_field_get_selection( opaque_pointer, 
						   $last_selection_start,
						   $last_selection_end )
      end
	 
   selection_start : INTEGER_16 is
	 --current starting byte offset of selected text
      do
	 get_selection
	 Result := last_selection_start
	 -- Result := c_form_field_get_selection( opaque_pointer, 0 )
      end
	 
   selection_end : INTEGER_16 is
	 --current ending byte offset of selected text
      do
	 get_selection
	 Result := last_selection_end
	 -- Result := c_form_field_get_selection( opaque_pointer, 1 )
      end
	 
   text_allocated_size : INTEGER_16 is
	 --number of bytes allocated to hold text (not necessarily 
	 --the string length)
      do
	 Result := c_form_field_get_text_allocated_size( opaque_pointer )
      end
	 
   text_handle : MEM_HANDLE is
	 --handle of the field's text.  There are some bugaboos with 
	 --using this after setting the field's text to a string 
	 --that's part of a database record; see PalmOS documentation
      do
	 create Result.from_handle(c_form_field_get_text_handle( opaque_pointer ))
      end
   
   text_height : INTEGER_16 is
	 --height in pixels of the number of lines which are not empty
      do
	 Result := c_form_field_get_text_height( opaque_pointer )
      end
   
   text_length : INTEGER_16 is
	 --length in bytes of the field's text
      do
	 Result := c_form_field_get_text_length( opaque_pointer )
      end
   
   visible_lines : INTEGER_16 is
	 --number of lines that can be displayed within the visible 
	 --bounds of a field
      do
	 Result := c_form_field_get_visible_lines( opaque_pointer )
      end
	 
   grab_focus is
	 --turns insertion point on and positions it within a field
      do
	 c_form_field_grab_focus( opaque_pointer )
      end
	 
   handle_event( event : EVENT ) is
      do
	 last_event_was_handled := c_form_field_handle_event( opaque_pointer, event.opaque_pointer )
      end
	 
   paste_from_clipboard is
      do
	 c_form_field_paste( opaque_pointer )
      end
	 
   recalculate_word_wrap is
	 --recalculates word-wrapping information and redraws the field
      do
	 c_form_field_recalculate( opaque_pointer, True )
      end
	 
   release_focus is
      do
	 c_form_field_release_focus( opaque_pointer )
      end
	 
   is_scrollable( direction : INTEGER_16 ) : BOOLEAN is
      do
	 Result := c_form_field_is_scrollable( opaque_pointer, direction )
      end
   
   Scroll_up : INTEGER_16 is
      external "C inline use <Field.h>"
      alias "winUp"
      end
   
   Scroll_down : INTEGER_16 is
      external "C inline use <Field.h>"
      alias "winDown"
      end
   
   basic_scroll( lines_to_scroll, direction : INTEGER_16 ) is
      do
	 c_form_field_scroll_field( opaque_pointer, lines_to_scroll, direction )
      end
   
   scroll( lines_to_scroll : INTEGER_16 ) is
	 --scrolls the appropriate number of lines; positive values for 
	 --lines_to_scroll scroll down; negative values scroll up
      do
	 if ( lines_to_scroll < 0 ) then
	    basic_scroll( -lines_to_scroll, Scroll_up )
	 else
	    basic_scroll( lines_to_scroll, Scroll_down )
	 end
      end
   
   scroll_pages( pages_to_scroll : INTEGER_16 ) is
	 --scrolls the number of "pages"; positive values for 
	 --pages_to_scroll scroll down; negative values scroll up
      do
	 scroll( (visible_lines - 1) * pages_to_scroll )
      end
	 
   set_is_dirty( new_is_dirty : BOOLEAN ) is
      do
	 c_form_field_set_dirty( opaque_pointer, new_is_dirty )
      end
	 
   set_font( new_font_id : INTEGER_16 ) is
      do
	 c_form_field_set_font( opaque_pointer, new_font_id )
      end
	 
   set_insertion_point( position : INTEGER_16 ) is
	 --sets the new insertion point for the field, but doesn't 
	 --necessarily make the new position visible
      do
	 c_form_field_set_insertion_point( opaque_pointer, position )
      end
	 
   set_insertion_point_position( position : INTEGER_16 ) is
	 --sets the insertion point and scrolls the field to make it visible
      do
	 c_form_field_set_insertion_point_position( opaque_pointer, position )
      end
	 
   set_max_chars( new_max_chars : INTEGER_16 ) is
	 --sets maximum number of bytes (including line-feeds) that 
	 --the field can accept
      do
	 c_form_field_set_max_chars( opaque_pointer, new_max_chars )
      end
	 
   set_scroll_position( new_position : INTEGER_16 ) is
	 --scrolls the field so that the character at the given position 
	 --is the first character on the first visible line
      do
	 c_form_field_set_scroll_position( opaque_pointer, new_position )
      end
	 
   set_selection( start_position, end_position : INTEGER_16 ) is
	 --sets the current selection in a field and highlights it
      do
	 c_form_field_set_selection( opaque_pointer, start_position, end_position )
      end
	 
   set_text_handle( new_handle : MEM_HANDLE ) is
	 --sets the text handle used by the field.  This does NOT 
	 --automatically free the previous handle, because the 
	 --previous handle might have been a pointer to memory you 
	 --don't want to free.  You must manually free the old handle 
	 --prior to setting a new one.
      do
	 c_form_field_set_text_handle( opaque_pointer, new_handle.handle )
      end
	 
   undo_last_change is
	 --undo the last change (typing, backspace, delete, paste, cut)
      do
	 c_form_field_undo( opaque_pointer )
      end
   
   contents : STRING is
      do
	 Result := text_handle.to_string
      end
   
   set_text_to_string( str : STRING ) is
	 --sets the text of the field to the given string.  This DOES
	 --free the old handle, so do NOT use this if the old handle was
	 --pointing to database memory, etc.
      local
	 old_handle : MEM_HANDLE
	 new_handle : MEM_HANDLE
      do
	 old_handle := text_handle 
	 create new_handle.allocate( str.count + 1, False )
	 new_handle.set_to_string( str )
	 set_text_handle( new_handle )
	 if old_handle.handle.is_not_null then
	    old_handle.release
	 end
      end
   
   
   
feature {NONE} --C access functions
   
   c_form_field_structure_size : INTEGER_32 is
      external "C inline use <Field.h>"
      alias "sizeof(FieldType)"
      end
   
   c_form_field_compact_text( p : POINTER ) is
      external "C use <Field.h>"
      alias "FldCompactText"
      end
   
   c_form_field_copy_selection_to_clipboard( p : POINTER ) is
      external "C use <Field.h>"
      alias "FldCopy"
      end
   
   c_form_field_cut_selection_to_clipboard( p : POINTER ) is
      external "C use <Field.h>"
      alias "FldCut"
      end
   
   c_form_field_delete_selection( p : POINTER; sel_start, sel_end : INTEGER_16 ) is
      external "C use <Field.h>"
      alias "FldDelete"
      end
   
   c_form_field_is_dirty( p : POINTER ) : BOOLEAN is
      external "C use <Field.h>" 
      alias "FldDirty"
      end
   
   c_form_field_draw_field( p : POINTER ) is
      external "C use <Field.h>"
      alias "FldDrawField"
      end
   
   c_form_field_erase_field( p : POINTER ) is
      external "C use <Field.h>"
      alias "FldEraseField"
      end
   
   c_form_field_free_memory( p : POINTER ) is
      external "C use <Field.h>"
      alias "FldFreeMemory"
      end
   
   c_form_field_get_font( p : POINTER ) : INTEGER_16 is
      external "C use <Field.h>"
      alias "FldGetFont"
      end
   
   c_form_field_get_insertion_point_position( p : POINTER ) : INTEGER_16 is
      external "C use <Field.h>"
      alias "FldGetInsPtPosition"
      end
   
   c_form_field_get_max_chars( p : POINTER ) : INTEGER_16 is
      external "C use <Field.h>"
      alias "FldGetMaxChars"
      end
   
   c_form_field_get_number_of_blank_lines( p : POINTER ) : INTEGER_16 is
      external "C use <Field.h>"
      alias "FldGetNumberOfBlankLines"
      end
   
   c_form_field_get_scroll_position( p : POINTER ) : INTEGER_16 is
      external "C use <Field.h>"
      alias "FldGetScrollPosition"
      end
   
   c_form_field_get_scroll_values( p,
				   pscroll_pos,
				   ptext_height,
				   pfield_height : POINTER ) is
      external "C use <Field.h>"
      alias "FldGetScrollValues"
      end
   
   c_form_field_get_selection( p, pstart, pend : POINTER ) : INTEGER_16 is
	 --gets selection values, passing a number through "val" to 
	 --indicate desired value.  0 = start; 1 = end
      external "C use <Field.h>"
      alias "FldGetSelection"
      end
   
   c_form_field_get_text_allocated_size( p : POINTER ) : INTEGER_16 is
      external "C use <Field.h>"
      alias "FldGetTextAllocatedSize"
      end
   
   c_form_field_get_text_handle( p : POINTER ) : POINTER is
      external "C use <Field.h>"
      alias "FldGetTextHandle"
      end
   
   c_form_field_get_text_height( p : POINTER ) : INTEGER_16 is
      external "C use <Field.h>"
      alias "FldGetTextHeight"
      end
   
   c_form_field_get_text_length( p : POINTER ) : INTEGER_16 is
      external "C use <Field.h>"
      alias "FldGetTextLength"
      end
   
   c_form_field_get_visible_lines( p : POINTER ) : INTEGER_16 is
      external "C use <Field.h>"
      alias "FldGetVisibleLines"
      end
   
   c_form_field_grab_focus( p : POINTER ) is
      external "C use <Field.h>"
      alias "FldGrabFocus"
      end
   
   c_form_field_handle_event( p, p_event : POINTER ) : BOOLEAN is
      external "C use <Field.h>"
      alias "FldHandleEvent"
      end
   
   c_form_field_paste( p : POINTER ) is
      external "C use <Field.h>"
      alias "FldPaste"
      end
   
   c_form_field_recalculate( p : POINTER; should_redraw : BOOLEAN ) is
      external "C use <Field.h>"
      alias "FldRecalculate"
      end
   
   c_form_field_release_focus( p : POINTER ) is
      external "C use <Field.h>"
      alias "FldReleaseFocus"
      end
   
   c_form_field_is_scrollable( p : POINTER; direction : INTEGER_16 ) : BOOLEAN is
      external "C use <Field.h>"
      alias "FldScrollable"
      end
   
   c_form_field_scroll_field( p : POINTER; lines_to_scroll, direction : INTEGER_16 ) is
      external "C use <Field.h>"
      alias "FldScrollField"
      end
   
   c_form_field_set_dirty( p : POINTER; new_is_dirty : BOOLEAN ) is
      external "C use <Field.h>"
      alias "FldSetDirty"
      end
   
   c_form_field_set_font( p : POINTER; new_font_id : INTEGER_16 ) is
      external "C use <Field.h>"
      alias "FldSetFont"
      end
   
   c_form_field_set_insertion_point( p : POINTER; position : INTEGER_16 ) is
      external "C use <Field.h>"
      alias "FldSetInsertionPoint"
      end
   
   c_form_field_set_insertion_point_position( p : POINTER; position : INTEGER_16 ) is
      external "C use <Field.h>"
      alias "FldSetInsPtPosition"
      end
   
   c_form_field_set_max_chars( p : POINTER; new_max_chars : INTEGER_16 ) is
      external "C use <Field.h>"
      alias "FldSetMaxChars"
      end
   
   c_form_field_set_scroll_position( p : POINTER; new_position : INTEGER_16 ) is
      external "C use <Field.h>"
      alias "FldSetScrollPosition"
      end
   
   c_form_field_set_selection( p : POINTER; start_position, end_position : INTEGER_16 ) is
      external "C use <Field.h>"
      alias "FldSetSelection"
      end
   
   c_form_field_set_text_handle( p, h : POINTER ) is
      external "C use <Field.h>"
      alias "FldSetTextHandle"
      end
   
   c_form_field_undo( p : POINTER ) is
      external "C use <Field.h>"
      alias "FldUndo"
      end
   
end

