indexing
   contents: "List access for form list objects"
   author: "Victor Putz, vputz@nyx.net"
   completed: no
   tested: no
   
class FORM_LIST
   
inherit
   FORM_OBJECT
      redefine
	 structure_size
      end
   EVENT_HANDLER
      redefine
	 handle_event
      end
   
creation   
   from_shared_memory, from_orphaned_memory, as_new_memory
   
feature {ANY}
   
   structure_size : INTEGER_32 is
      do
	 Result := c_form_list_structure_size
      end
   
   draw is
      do
	 c_form_list_draw_list( opaque_pointer )
      end
	 
   erase is
      do
	 c_form_list_erase_list( opaque_pointer )
      end
	 
   number_of_items : INTEGER_16 is
      do
	 Result := c_form_list_get_number_of_items( opaque_pointer )
      end
	 
   current_selection : INTEGER_16 is
      do
	 Result := c_form_list_get_selection( opaque_pointer )
      end
   
   selection_text( item_number : INTEGER_16 ) : STRING is
      do
	 create Result.from_external_copy( c_form_list_get_selection_text( opaque_pointer, item_number ))
      end
					   
   current_selection_text : STRING is
      do
	 Result := selection_text( current_selection )
      end
	 
   number_of_visible_items : INTEGER_16 is
      do
	 Result := c_form_list_get_number_of_visible_items( opaque_pointer )
      end
	 
   handle_event( event : EVENT ) is
      do
	 last_event_was_handled := c_form_list_handle_event( opaque_pointer, event.opaque_pointer )
      end
	 
   make_item_visible( item_number : INTEGER_16 ) is
	 --makes an item visible, preferably at the top of the list
      do
	 c_form_list_make_item_visible( opaque_pointer, item_number )
      end
   
   last_scroll_was_successful : BOOLEAN
   
   scroll( direction, item_count : INTEGER_16 ) is
      do
	 last_scroll_was_successful := c_form_list_scroll_list( opaque_pointer, direction, item_count )
      end
	 
   set_number_of_visible_items( new_visible_items : INTEGER_16 ) is
      do
	 c_form_list_set_height( opaque_pointer, new_visible_items )
      end
   
   pointer_array_from_string_array( strings : ARRAY[ STRING ] ) : ARRAY[ POINTER ] is
	 --creates a c-compatible array of pointers to strings.  This 
	 --seems to be necessary when using set_list_choices because 
	 --the list only keeps that pointer--so the array must be 
	 --kept "live" rather than being created and destroyed on the 
	 --fly.  More testing is probably needed on the interaction 
	 --between this and set_list_choices.  Consider adding a 
	 --list_choices array member to FORM_LIST--but this will 
	 --require maintaining a FORM_LIST object rather than 
	 --accessing it by reference.
      local
	 i : INTEGER_16
      do
	 create Result.with_capacity( strings.count, 0 )
	 from
	    i := strings.lower.to_integer_16
	 until 
	    not strings.valid_index(i)
	 loop
	    Result.add_last( strings.item(i).to_external )
	    i := i + 1
	 end
      end
   
   set_list_choices_to_pointer_array( choices : ARRAY[POINTER] ) is
	 --sets the items of a list to the array of pointers to 
	 --strings.  Use this by keeping two arrays: first, an array 
	 --of strings (ARRAY[STRING]).  Convert this to an array of 
	 --pointers using FORM_LIST.pointer_array_from_string_array.  
	 --Finally, pass this ARRAY[POINTER] to this feature.  You 
	 --must keep both the ARRAY[STRING] and the ARRAY[POINTER] 
	 --alive as long as you are using the PalmOS list, because it 
	 --only stores a pointer to the ARRAY[POINTER], which in turn 
	 --is pointing to the ARRAY[STRING]--so don't change either 
	 --of those without resetting the list pointer, or it will 
	 --corrupt the list choices and be rather tricky to hunt down.
      do
	 c_form_list_set_list_choices( opaque_pointer, choices.to_external, choices.count.to_integer_16 )
      end
   
   set_list_choices( choices : ARRAY[ STRING ] ) is
	 --sets the items of a list to the given array of text strings
      do
	 --create an array of pointers to c-compatible strings
	 c_form_list_set_list_choices( opaque_pointer, pointer_array_from_string_array( choices ).to_external, choices.count.to_integer_16 )
      end
   
   set_position( new_x, new_y : INTEGER_16 ) is
	 --list is not redrawn; don't call if list is visible
      do
	 c_form_list_set_position( opaque_pointer, new_x, new_y )
      end
   
   set_selection( new_selection : INTEGER_16 ) is
	 --selects new selection and scrolls/redraws as necessary
      do
	 c_form_list_set_selection( opaque_pointer, new_selection )
      end
   
   set_top_item( new_top_item : INTEGER_16 ) is
	 --sets the new "top item" on the list.  Item cannot become 
	 --the top item if it's on the last page of the list
      do
	 c_form_list_set_top_item( opaque_pointer, new_top_item )
      end
   
feature {NONE} --C functions
   
   c_form_list_structure_size : INTEGER_32 is
      external "C inline use <List.h>"
		alias "(sizeof(ListType))"
      end
   
   c_form_list_draw_list( p : POINTER ) is
      external "C use <List.h>"
      alias "LstDrawList"
      end
   
   c_form_list_erase_list( p : POINTER ) is
      external "C use <List.h>"
      alias "LstEraseList"
      end
   
   c_form_list_get_number_of_items( p : POINTER ) : INTEGER_16 is
      external "C use <List.h>"
      alias "LstGetNumberOfItems"
      end
   
   c_form_list_get_selection( p : POINTER ) : INTEGER_16 is
      external "C use <List.h>"
      alias "LstGetSelection"
      end
   
   c_form_list_get_selection_text( p : POINTER; item_number : integer ) : POINTER is
      external "C use <List.h>"
      alias "LstGetSelectionText"
      end
   
   c_form_list_get_number_of_visible_items( p : POINTER ) : INTEGER_16 is
      external "C use <List.h>"
      alias "LstGetVisibleItems"
      end
   
   c_form_list_handle_event( p_list, p_event : POINTER ) : BOOLEAN is
      external "C use <List.h>"
      alias "LstHandleEvent"
      end
   
   c_form_list_make_item_visible( p : POINTER; item_number : INTEGER_16 ) is
      external "C use <List.h>"
      alias "LstMakeItemVisible"
      end
   
   c_form_list_scroll_list( p : POINTER; direction, item_count : INTEGER_16 ) : BOOLEAN is
      external "C use <List.h>"
      alias "LstScrollList"
      end
   
   c_form_list_set_height( p : POINTER; new_visible_items : INTEGER_16 ) is
      external "C use <List.h>"
      alias "LstSetHeight"
      end
   
   c_form_list_set_list_choices( p_list, p_choice_array : POINTER; num_items : INTEGER_16 ) is
      external "C use <List.h>"
      alias "LstSetListChoices"
      end
   
   c_form_list_set_position( p : POINTER; new_x, new_y : INTEGER_16 ) is
      external "C use <List.h>"
      alias "LstSetPosition"
      end
   
   c_form_list_set_selection( p : POINTER; new_selection : INTEGER_16 ) is
      external "C use <List.h>"
      alias "LstSetSelection"
      end
   
   c_form_list_set_top_item( p : POINTER; new_top_item : INTEGER_16 ) is
      external "C use <List.h>"
      alias "LstSetTopItem"
      end
   
end

   
