

indexing
   contents: "generated wrapper for C type RectangleType"
   author: "Victor B. Putz"
   completed: no
   tested: no
   

class PALM_RECTANGLE
inherit
   C_STRUCT
      redefine
	 structure_size
      end

   
creation
   from_shared_memory, from_orphaned_memory, as_new_memory
   
feature {ANY} -- direct Eiffel features

   structure_size : INTEGER_32 is
      do
         Result := c_palm_rectangle_structure_size
      end

   c_palm_rectangle_structure_size : INTEGER_32 is 
      external "C inline use <Rect.h>" 
      alias "(sizeof(RectangleType))"
      end

   palmos_copy : PALM_RECTANGLE is
      do
         create Result.as_new_memory
         c_rct_copy_rectangle( opaque_pointer, Result.opaque_pointer )
      end


   intersection_with( other : PALM_RECTANGLE ) : PALM_RECTANGLE is
      do
         create Result.as_new_memory
         c_rct_get_intersection( opaque_pointer, other.opaque_pointer, Result.opaque_pointer )
      end


   right : INTEGER_16 is do Result := left + width end

   bottom : INTEGER_16 is do Result := top + height end
   

   draw_as_grey_frame( frame_type : INTEGER_16 ) is
      do
	 c_palm_rectangle_draw_as_grey_frame( frame_type, opaque_pointer )
      end
   
   
   draw_frame( frame_type : INTEGER_16 ) is
      do
	 c_palm_rectangle_draw_frame( frame_type, opaque_pointer )
      end

   
   erase_frame( frame_type : INTEGER_16 ) is
      do
	 c_palm_rectangle_erase_frame( frame_type, opaque_pointer )
      end


   frame_rectangle( frame_type : INTEGER_16 ) : like Current is
      do
         create Result.as_new_memory
         c_palm_rectangle_frame_rectangle_base( frame_type, opaque_pointer, Result.opaque_pointer )
      end


   invert_frame( frame_type : INTEGER_16 ) is
      do
	 c_palm_rectangle_invert_frame( frame_type, opaque_pointer )
      end

   
   paint_frame( frame_type : INTEGER_16 ) is
      do
	 c_palm_rectangle_paint_frame( frame_type, opaque_pointer )
      end

   
   scroll( direction, distance : INTEGER_16 ) : like Current is
      do
         create Result.as_new_memory
         c_palm_rectangle_scroll_rectangle( opaque_pointer, direction, distance, Result.opaque_pointer )
      end


feature {ANY} -- structure features (aliased)
   inset_by (inset_amount : INTEGER_16)  is
      do
         c_alias_inset_by( opaque_pointer, inset_amount )
      end


   offset_by (x : INTEGER_16; y : INTEGER_16)  is
      do
         c_alias_offset_by( opaque_pointer, x, y )
      end


   point_is_in_rectangle (x : INTEGER_16; y : INTEGER_16) : BOOLEAN is
      do
         Result := c_alias_point_is_in_rectangle( opaque_pointer, x, y )
      end


   set (new_left : INTEGER_16; new_y : INTEGER_16; new_width : INTEGER_16; new_height : INTEGER_16)  is
      do
         c_alias_set( opaque_pointer, new_left, new_y, new_width, new_height )
      end


   clip_to_drawing_window   is
      do
         c_alias_clip_to_drawing_window( opaque_pointer )
      end


   draw (corner_diameter : INTEGER_16)  is
      do
         c_alias_draw( opaque_pointer, corner_diameter )
      end


   erase (corner_diameter : INTEGER_16)  is
      do
         c_alias_erase( opaque_pointer, corner_diameter )
      end


   fill (corner_diameter : INTEGER_16)  is
      do
         c_alias_fill( opaque_pointer, corner_diameter )
      end


   from_clipping_rectangle   is
      do
         c_alias_from_clipping_rectangle( opaque_pointer )
      end


   from_window_bounds   is
      do
         c_alias_from_window_bounds( opaque_pointer )
      end


   invert   is
      do
         c_alias_invert( opaque_pointer )
      end


   paint (corner_diameter : INTEGER_16)  is
      do
         c_alias_paint( opaque_pointer, corner_diameter )
      end



feature {ANY} -- aliased helper features for above
   c_alias_inset_by (p : POINTER; inset_amount : INTEGER_16)  is
      external "C use <Rect.h>"
      alias "RctInsetRectangle"
      end

   c_alias_offset_by (p : POINTER; x, y : INTEGER_16)  is
      external "C use <Rect.h>"
      alias "RctOffsetRectangle"
      end

   c_alias_point_is_in_rectangle (p : POINTER; x, y : INTEGER_16) : BOOLEAN is
      external "C use <Rect.h>"
      alias "RctPtInRectangle"
      end

   c_alias_set (p : POINTER; 
		new_left, new_top, new_width, new_height : INTEGER_16 )  is
      external "C use <Rect.h>"
      alias "RctSetRectangle"
      end

   c_alias_clip_to_drawing_window (p : POINTER)  is
      external "C use <Window.h>"
      alias "WinClipRectangle"
      end

   c_alias_draw (p : POINTER; corner_diameter : INTEGER_16)  is
      external "C use <Window.h>"
      alias "WinDrawRectangle"
      end

   c_alias_erase (p : POINTER; corner_diameter : INTEGER_16)  is
      external "C use <Window.h>"
      alias "WinEraseRectangle"
      end

   c_alias_fill (p : POINTER; corner_diameter : INTEGER_16)  is
      external "C use <Window.h>"
      alias "WinFillRectangle"
      end

   c_alias_from_clipping_rectangle (p : POINTER)  is
      external "C use <Window.h>"
      alias "WinGetClip"
      end

   c_alias_from_window_bounds (p : POINTER)  is
      external "C use <Window.h>"
      alias "WinGetWindowBounds"
      end

   c_alias_invert (p : POINTER)  is
      external "C use <Window.h>"
      alias "WinInvertRectangle"
      end

   c_alias_paint (p : POINTER; corner_diameter : INTEGER_16)  is
      external "C use <Window.h>"
      alias "WinPaintRectangle"
      end


feature {ANY} -- aliased C features
   c_rct_copy_rectangle (p_source, p_dest : POINTER)  is
      external "C use <Rect.h>"
      alias "RctCopyRectangle"
      end

   c_rct_get_intersection (p, p_other, p_dest : POINTER ) is
      external "C use <Rect.h>"
      alias "RctGetIntersection"
      end

   c_palm_rectangle_draw_as_grey_frame (frame_type : INTEGER_16; 
					p : POINTER)  is
      external "C use <Window.h>"
      alias "WinDrawGrayRectangleFrame"
      end

   c_palm_rectangle_draw_frame (frame_type : INTEGER_16; 
				p : POINTER)  is
      external "C use <Window.h>"
      alias "WinDrawRectangleFrame"
      end

   c_palm_rectangle_erase_frame (frame_type : INTEGER_16; 
				 p : POINTER)  is
      external "C use <Window.h>"
      alias "WinEraseRectangleFrame"
      end

   c_palm_rectangle_frame_rectangle_base (frame_type : INTEGER_16; 
					  p_source, p_dest : POINTER)  is
      external "C use <Window.h>"
      alias "WinGetFrameRectangle"
      end

   c_palm_rectangle_invert_frame (frame_type : INTEGER_16; 
				  p : POINTER)  is
      external "C use <Window.h>"
      alias "WinInvertRectangle"
      end

   c_palm_rectangle_paint_frame (frame_type : INTEGER_16; 
				 p : POINTER)  is
      external "C use <Window.h>"
      alias "WinPaintRectangleFrame"
      end

   c_palm_rectangle_scroll_rectangle (p : POINTER; 
				      direction, distance : INTEGER_16; 
				      p_result : POINTER)  is
      external "C use <Window.h>"
      alias "WinScrollRectangle"
      end


feature {ANY} -- data features
   left : INTEGER_16 is do
      Result := c_palm_rectangle_left( opaque_pointer ) 
   end

   top : INTEGER_16 is do
      Result := c_palm_rectangle_top( opaque_pointer ) 
   end

   width : INTEGER_16 is do
      Result := c_palm_rectangle_width( opaque_pointer ) 
   end

   height : INTEGER_16 is do
      Result := c_palm_rectangle_height( opaque_pointer ) 
   end


feature {ANY} -- c_with_pointer features

feature {ANY} -- c_without_pointer features

feature {NONE} -- data feature support functions
   c_palm_rectangle_left( p : POINTER ) : INTEGER_16 is 
      external "C struct RectangleType get topLeft.x use <Rect.h>" 
      end

   c_palm_rectangle_top( p : POINTER ) : INTEGER_16 is 
      external "C struct RectangleType get topLeft.y use <Rect.h>" 
      end

   c_palm_rectangle_width( p : POINTER ) : INTEGER_16 is 
      external "C struct RectangleType get extent.x use <Rect.h>" 
      end

   c_palm_rectangle_height( p : POINTER ) : INTEGER_16 is
      external "C struct RectangleType get extent.y  use <Rect.h>"
      end

end
