
indexing
   contents: "a typed database which stores records of a specific type"
   author: "Victor B. Putz"
   completed: no
   tested: no
   
class RECORD_DATABASE
   
inherit
   DATABASE
   
creation
   from_shared_memory, from_orphaned_memory, as_new_memory, from_database
   
feature {ANY}
   
   from_database( d : DATABASE ) is
      do
	 from_shared_memory( d.opaque_pointer )
      end
   
   set_typed_record_from_index( index : INTEGER_16; record : SIMPLE_DATABASE_RECORD ) is
	 --copies the contents of the database record at the given 
	 --index into a new object of type SIMPLE_DATABASE_RECORD, then returns that object
      require
	 is_valid_index( index )
      local
	 h : MEM_HANDLE
      do
	 h := query_record( index )
	 h.lock
	 record.read_from_handle( h )
	 h.unlock
      end
   
   new_typed_record( index : INTEGER_16; record : SIMPLE_DATABASE_RECORD ) is
	 --inserts the new record into the database at the given 
	 --index if possible.  The actual index used is stored at 
	 --last_new_record_index
      local
	 h : MEM_HANDLE
      do
	 h := new_record( index, record.record_size )
	 record.dm_write_to_handle( h )
	 release_record_at_index( last_new_record_index, True )
      end
   
   write_typed_record_to_index( index : INTEGER_16; record : SIMPLE_DATABASE_RECORD ) is
	 --writes the contents of record into the record at index.  
	 --This will resize the record at index if necessary.
      require
	 is_valid_index( index )
      local
	 h : MEM_HANDLE
      do
	 h := resize_record( index, record.record_size )
	 record.dm_write_to_handle( h )
	 release_record_at_index( index, True )
      end
   
   application_info_is_defined : BOOLEAN is
      do
	 Result := application_info_id.is_zero
      end
   
   set_to_application_info( info : SIMPLE_DATABASE_RECORD ) is
      require
	 application_info_is_defined
      local
	 p : POINTER
      do
	 get_info
	 p := application_info_id.to_locked_pointer( info_card_number )
	 info.read_from_pointer( p )
	 mem_ptr_unlock( p )
      end
   
   mem_ptr_unlock( p : POINTER ) is
      external "C inline use <MemoryMgr.h>"
      alias "MemPtrUnlock"
      end
   
   remove_application_info is
      local
	 null_local_id : LOCAL_ID
      do
	 null_local_id.set_low_word( 0 )
	 null_local_id.set_high_word( 0 )
	 get_info
	 database_manager.set_database_application_info( info_card_number, info_local_id, null_local_id )
      end
   
   set_application_info( new_application_info : SIMPLE_DATABASE_RECORD ) is
      local
	 h : MEM_HANDLE
      do
	 if application_info_is_defined then
	    remove_application_info
	 end
	 h := new_handle( new_application_info.record_size_uint32 )
	 get_info
	 database_manager.set_database_application_info( info_card_number, info_local_id, h.to_local_id )
--	 h.lock
--	 new_application_info.dm_write_to_pointer( h.locked_pointer, 0 )
--	 h.unlock
      end
   
end
	 
