
class Data_element

  def initialize( name, type ) 
    @name = name
    @type = type
  end

  attr_accessor :name, :type

  def eiffel_feature_text
    "   #{@name} : #{@type}\n   set_#{name} (new_value : #{type}) is\n      do\n         #{@name} := new_value\n      end\n\n"
  end

  def size_text
    case @type
    when "INTEGER_16" then "sizeof_int16"
    when "CHARACTER" then "sizeof_character"
    when "STRING" then "sizeof_string( #{@name} )"
    when "UINT32" then "sizeof_uint32"
    else "sizeof_USERDEFINED"
    end
  end

  def read_text
    result =  case @type
	      when "INTEGER_16" then "         #{@name} := read_int16( cursor )\n"
	      when "CHARACTER" then "         #{@name} := read_character( cursor )\n"
	      when "STRING" then "         #{@name} := read_string( cursor )\n"
              when "UINT32" then "         #{@name} := read_uint32( cursor )\n"
	      else "         read_USERDEFINED( cursor, #{@name} )\n"
	      end
    result + "         cursor := pointer_plus( cursor, #{size_text} )\n"
  end

  def write_text
    result =  case @type
	      when "INTEGER_16" then "         write_int16( cursor, #{@name} )\n"
	      when "CHARACTER" then "         write_character( cursor, #{@name} )\n"
	      when "STRING" then "         write_string( cursor, #{@name} )\n"
              when "UINT32" then "         write_uint32( cursor, #{@name} )\n"
	      else "         write_USERDEFINED( cursor, #{@name} )\n"
	      end
    result + "         cursor := pointer_plus( cursor, #{size_text} )\n"
  end

  def dm_write_text
    result =  case @type
	      when "INTEGER_16" then "         dm_write_int16( p, offset, #{@name} )\n"
	      when "CHARACTER" then "         dm_write_character( p, offset, #{@name} )\n"
	      when "STRING" then "         dm_write_string( p, offset, #{@name} )\n"
              when "UINT32" then "         dm_write_uint32( p, offset, #{@name} )\n"
	      else "         dm_write_USERDEFINED( p, offset, #{@name} )\n"
	      end
    result + "         offset := offset + #{size_text}\n"
  end

end

class Record_wrapper_factory

  def each_match( str, re )
    result = []
    m = re.match( str )
    while m != nil
      result << yield( m[0] )
      m = re.match( m.post_match )
    end
    result
  end

  Data_element_regex = /([A-Za-z_]+)\s*:\s*([A-Za-z0-9_]+)/

  def data_element_from_text( str )
    m = Data_element_regex.match( str )
    if m != nil then
      Data_element.new( m[1], m[2] )
    end
  end

  def data_elements_from_text( str ) 
    each_match( str, Data_element_regex ) { |m|
      data_element_from_text( m )
    }
  end

  Header_regex = Regexp.compile( /simple_database_record\s*([A-Za-z_]+)\s*is/ )

  def class_name( str )
    m = Header_regex.match( str ) 
    if m != nil then
      m[1]
    end
  end

end

class Record_wrapper

  def initialize( str )
    c = Record_wrapper_factory.new
    @data_elements = c.data_elements_from_text( str )
    @class_name = c.class_name( str )
  end

  attr_accessor :class_name, :data_elements

  def size_feature_text
    result = "
   record_size : INTEGER is
      do
         Result := 0\n"
    data_elements.each { |i|
      result << "         Result := Result + #{i.size_text}\n"
    }
    result << "      end\n"
    result
  end

  def read_feature_text
    result = "
   read_from_pointer( p : POINTER ) is
      local
         cursor : POINTER
      do
         cursor := p\n"
    data_elements.each { |i|
      result << i.read_text
    }
    result << "      end\n\n"
    result
  end

  def write_feature_text
    result = "
   write_to_pointer( p : POINTER ) is
      local
         cursor : POINTER
      do
         cursor := p\n"
    @data_elements.each { |i| 
      result << i.write_text
    }
    result << "      end\n\n"
    result
  end

  def dm_write_feature_text
    result = "
   dm_write_to_pointer( p : POINTER; starting_offset : INTEGER ) is
      local
        offset : INTEGER
      do
        offset := starting_offset"
    @data_elements.each { |i|
      result << i.dm_write_text
    }
    result << "      end\n\n"
  end

  def data_element_accessors_text
    result = ""
    @data_elements.each { |i|
      result << i.eiffel_feature_text
    }
    result
  end
 
  def class_text
    "
class #{@class_name}

inherit
   SIMPLE_DATABASE_RECORD

feature {ANY} -- basic data elements

#{data_element_accessors_text}
feature {ANY} -- total record size

#{size_feature_text}

feature {ANY} -- class reading/writing features

#{read_feature_text}
#{write_feature_text}
#{dm_write_feature_text}

end -- class #{@class_name}
"
end
end

if ARGV.length == 1
  File.open( ARGV[0], "r" ) { |infile|
    s = infile.read
    w = Record_wrapper.new( s )
    File.open( ARGV[0].gsub( /\.ear/, ".e" ), "w" ) { |classfile|
      classfile.write( w.class_text )
    }
  }
else
  print "wrong arguments:\n\nUsage: make_simple_record interface_file\n\n"
end

