class Data_feature
   def initialize( eiffel_name, eiffel_type, c_type, c_name )
     @eiffel_name = eiffel_name
     @eiffel_type = eiffel_type
     @c_type = c_type
     @c_name = c_name
   end
   
   def c_feature_name( classname )
     "c_#{classname.downcase}_#{@eiffel_name.downcase}"
   end
   
   def eiffel_text( classname )
     "   #@eiffel_name : #@eiffel_type is do\n      Result := #{c_feature_name(classname)}( opaque_pointer ) \n   end\n" 
   end

   def c_feature_text( classname )
     "   #{c_feature_name(classname)}( p : POINTER ) : #{@eiffel_type} is external \"C\" end\n"
   end

   def c_text( classname, c_pointer_type )
     "#{@c_type}\n#{c_feature_name(classname)} ( EIF_POINTER p )\n{\n  return (#{@c_type})(((#{c_pointer_type})(p))->#{@c_name});\n}\n"
   end

 end

 class Parameter
   def initialize (name, type)
     @name = name
     @type = type
   end
   
   attr_reader :name, :type
 end

module Eiffel_printer
  def eiffel_parameters_text( parm_array )
     if parm_array.length == 0 then ""
     else
       result = "("
       parm_array.each { |parm|
	 result << "#{parm.name} : #{parm.type}; "
       }
       result.chomp!( "; " )
       result << ")"
       result
     end
  end

  def eiffel_feature_return_type_text( type_string )
    if type_string == "void" then
      ""
    else
      ": #{type_string}"
    end
  end

end


class C_alias_feature

  include Eiffel_printer

  def initialize( eiffel_name, eiffel_parameters, eiffel_type, c_name )
    @eiffel_name = eiffel_name
    @eiffel_parameters = eiffel_parameters
    @eiffel_type = eiffel_type
    @c_name = c_name
  end

  def eiffel_feature_type
    eiffel_feature_return_type_text( @eiffel_type )
  end

  def eiffel_parameter_list
    eiffel_parameters_text( @eiffel_parameters )
  end

  def eiffel_text
    "   #{@eiffel_name} #{eiffel_parameter_list} #{eiffel_feature_type} is\n      external \"C_InlineWithoutCurrent\"\n      alias \"#{@c_name}\"\n      end\n"
  end
  
end

class C_structured_alias_feature < C_alias_feature
  
  def eiffel_support_alias_feature_name
    "c_alias_#{@eiffel_name}"
  end

  def eiffel_support_alias_feature_parameters
    [ Parameter.new( "p", "POINTER" ) ] + @eiffel_parameters
  end
  
  def eiffel_alias_call_parameters_text
    result = "( opaque_pointer, "
    @eiffel_parameters.each { |p|
      result << "#{p.name}, "
    }
    result.chomp!( ", " )
    result << " )"
    result
  end
  
  def eiffel_body_text
    if @eiffel_type == "void" then
      "#{eiffel_support_alias_feature_name}#{eiffel_alias_call_parameters_text}"
    else
      "Result := #{eiffel_support_alias_feature_name}#{eiffel_alias_call_parameters_text}"
    end
  end

  def eiffel_text
    "   #{@eiffel_name} #{eiffel_parameter_list} #{eiffel_feature_type} is\n      do\n         #{eiffel_body_text}\n      end\n\n"
  end

  def eiffel_support_alias_feature_text
    "   #{eiffel_support_alias_feature_name} #{eiffel_parameters_text( eiffel_support_alias_feature_parameters )} #{eiffel_feature_type} is\n      external \"C_InlineWithoutCurrent\"\n      alias \"#{@c_name}\"\n      end\n"
  end

end

 class C_without_pointer_feature
   include Eiffel_printer
   def initialize( eiffel_name, eiffel_parameters, eiffel_type, c_type, c_parameters, c_code )
     @eiffel_name = eiffel_name
     @eiffel_parameters = eiffel_parameters
     @eiffel_type = eiffel_type
     @c_type = c_type
     @c_parameters = c_parameters
     @c_code = c_code
   end
   
   def c_feature_name( classname )
     "c_#{classname.downcase}_#{@eiffel_name.downcase}"
   end

   def c_feature_parameter_list
     result = ""
     if @eiffel_parameters.length != 0 then
       result = "( "
       @eiffel_parameters.each { |parm|
	 result << "#{parm.name} : #{parm.type}; "
       }
       result.chomp!( "; " )
       result << " )"
     end
   end
   
   def c_feature_return_type
     if @eiffel_type == "void" then "" else " : #{@eiffel_type}" end
   end
   
   def c_parameter_list
     result = ""
     if @c_parameters.length != 0 then
       @c_parameters.each{ |parm|
	 result << "#{parm.type} #{parm.name}, "
       }
     end
     result.chomp!( ", " )
   end

   def c_text( classname )
     "#{@c_type}\n#{c_feature_name( classname )} (#{c_parameter_list})\n{\n#{@c_code}\n}\n"
   end

   def c_feature_text( classname )
     "   #{c_feature_name(classname)} #{c_feature_parameter_list} #{c_feature_return_type} is external \"C\" end\n"
   end

   def eiffel_feature_type
     eiffel_feature_return_type_text( @eiffel_type )
   end

   def eiffel_feature_body( classname )
     result = ""
     if @eiffel_type != "void" then result = "Result := " end
     result << c_feature_name( classname )
     if @eiffel_parameters.length != 0 then 
       result << "( "
       @eiffel_parameters.each { |parm|
	 result << "#{parm.name}, "
       }
       result.chomp!( ", ")
       result << " )"
     end
     result
   end

   def eiffel_parameter_list
     eiffel_parameters_text( @eiffel_parameters )
   end

  def eiffel_text( classname )
    "   #{@eiffel_name} #{eiffel_parameter_list} #{eiffel_feature_type} is\n      do\n         #{eiffel_feature_body(classname)}\n      end\n"
  end
  
end

 class C_with_pointer_feature
   include Eiffel_printer

   def initialize( eiffel_name, eiffel_parameters, eiffel_type, c_type, c_parameters, c_code )
     @eiffel_name = eiffel_name
     @eiffel_parameters = eiffel_parameters
     @eiffel_type = eiffel_type
     @c_type = c_type
     @c_parameters = c_parameters
     @c_code = c_code
   end
   
   def c_feature_name( classname )
     "c_#{classname.downcase}_#{@eiffel_name.downcase}"
   end

   def c_feature_parameter_list
     result = "( p : POINTER; "
     @eiffel_parameters.each { |parm|
       result << "#{parm.name} : #{parm.type}; "
     }
     result.chomp!( "; " )
     result << " )"
   end
   
   def c_feature_return_type
     if @eiffel_type == "void" then "" else " : #{@eiffel_type}" end
   end
   
   def c_parameter_list
     result = "EIF_POINTER p, "
     if @c_parameters.length != 0 then
       @c_parameters.each{ |parm|
	 result << "#{parm.type} #{parm.name}, "
       }
     end
     result.chomp!( ", " )
   end

   def c_text( classname )
     "#{@c_type}\n#{c_feature_name( classname )} (#{c_parameter_list})\n{\n#{@c_code}\n}\n"
   end

   def c_feature_text( classname )
     "   #{c_feature_name(classname)} #{c_feature_parameter_list} #{c_feature_return_type} is external \"C\" end\n"
   end

   def eiffel_feature_type
     eiffel_feature_return_type_text( @eiffel_type )
   end

   def eiffel_feature_body( classname )
     result = ""
     if @eiffel_type != "void" then result = "Result := " end
     result << c_feature_name( classname )
     result << "( opaque_pointer, "
     if @eiffel_parameters.length != 0 then
       @eiffel_parameters.each { |parm|
	 result << "#{parm.name}, "
       }
     end
     result.chomp!( ", ")
     result << " )"
     result
   end

   def eiffel_parameter_list
     eiffel_parameters_text( @eiffel_parameters )
   end

  def eiffel_text( classname )
    "   #{@eiffel_name} #{eiffel_parameter_list} #{eiffel_feature_type} is\n      do\n         #{eiffel_feature_body(classname)}\n      end\n"
  end
  
end

class Text_feature

  def initialize( text ) 
    @text = text
  end
  
  attr_reader :text
end

class Wrapper_factory

  def each_match( str, re )
    result = []
    m = re.match( str )
    while m != nil
      result << yield( m[0] )
      m = re.match( m.post_match )
    end
    result
  end

  Data_feature_regex = /([A-Za-z_0-9]+)\s*:\s*([A-Za-z_]+)\s*is\s*([A-Za-z_]+)\s*([A-Za-z0-9_\.]+)\s*/

  def data_feature_from_text( str )
    m = Data_feature_regex.match( str )
    if m != nil then
      Data_feature.new( m[1], m[2], m[3], m[4] )
    end
  end

  def data_features_from_text( str ) 
    each_match( str, Data_feature_regex ) { |m|
      data_feature_from_text( m )
    }
  end

  def strip_data_features( str )
    str.gsub!( Data_feature_regex, "" )
  end

  def parameters_from_string( str )
    re = /([A-Za-z_0-9]+)\s*:\s*([A-Za-z_]+)/
    parmchunks = str.split( "\s*;\s*" )
    result = []
    parmchunks.each { |parm|
      m = re.match( parm )
      if m != nil then
	result << Parameter.new( m[1], m[2] )
      end	
    }
    result
  end

  C_with_pointer_feature_regex = Regexp.compile( /([A-Za-z_0-9]+)\s*\((.*?)\)\s*:\s*([A-Za-z_]+)\s*is\s*([A-Za-z_]+)\s*\((.*?)\)\s*\{\s*(.*?)\}\s*/ )

  def c_with_pointer_feature_from_text( str )
    m = C_with_pointer_feature_regex.match( str )
    if m != nil then
      C_with_pointer_feature.new( m[1], parameters_from_string( m[2] ),
				 m[3], m[4], parameters_from_string( m[5] ),
				 m[6] )
    end
  end

  def c_with_pointer_features_from_text( str )
    each_match( str, C_with_pointer_feature_regex ) { |m|
      c_with_pointer_feature_from_text( m )
    }
  end

  def strip_c_with_pointer_features( str )
    str.gsub!( C_with_pointer_feature_regex, "" )
  end

  C_without_pointer_feature_regex = Regexp.compile( /([A-Za-z_0-9]+)\s*\((.*?)\)\s*:\s*([A-Za-z_]+)\s*wraps\s*([A-Za-z_]+)\s*\((.*?)\)\s*\{\s*(.*?)\}\s*/ )

  def c_without_pointer_feature_from_text( str )
    m = C_without_pointer_feature_regex.match( str )
    if m != nil then
      C_without_pointer_feature.new( m[1], parameters_from_string( m[2] ),
				 m[3], m[4], parameters_from_string( m[5] ),
				 m[6] )
    end
  end

  def c_without_pointer_features_from_text( str )
    each_match( str, C_without_pointer_feature_regex ) { |m|
      c_without_pointer_feature_from_text( m )
    }
  end

  def strip_c_without_pointer_features( str )
    str.gsub!( C_without_pointer_feature_regex, "" )
  end

  C_alias_feature_regex = Regexp.compile( /([A-Za-z_0-9]+)\s*\((.*?)\)\s* : \s*([A-Za-z_]+)\s*aliases\s*([A-Za-z_]+)\s*/ )

  def c_alias_feature_from_text( str ) 
    m = C_alias_feature_regex.match( str )
    if m != nil then
      C_alias_feature.new( m[1], parameters_from_string( m[2] ), m[3], m[4] )
    end
  end

  def c_alias_features_from_text( str )
    each_match( str, C_alias_feature_regex ) { |m|
      c_alias_feature_from_text( m )
    }
  end

  def strip_c_alias_features( str )
    str.gsub!( C_alias_feature_regex, "" )
  end

  C_alias_structure_feature_regex = Regexp.compile( /([A-Za-z_0-9]+)\s*\((.*?)\)\s* : \s*([A-Za-z_]+)\s*aliases_structure_call\s*([A-Za-z_]+)\s*/ )

  def c_alias_structure_feature_from_text( str ) 
    m = C_alias_structure_feature_regex.match( str )
    if m != nil then
      C_structured_alias_feature.new( m[1], parameters_from_string( m[2] ), m[3], m[4] )
    end
  end

  def c_alias_structure_features_from_text( str )
    each_match( str, C_alias_structure_feature_regex ) { |m|
      c_alias_structure_feature_from_text( m )
    }
  end

  def strip_c_alias_structure_features( str )
    str.gsub!( C_alias_structure_feature_regex, "" )
  end

  Eiffel_feature_regex = Regexp.compile( /<eiffel>(.*?)<\/eiffel>/, Regexp::MULTILINE )

  Inherit_block_regex = Regexp.compile( /<inherit>(.*?)<\/inherit>/, Regexp::MULTILINE )

  C_feature_regex = Regexp.compile( /<C>(.*?)<\/C>/, Regexp::MULTILINE )

  C_include_regex = Regexp.compile( /<C_include>(.*?)<\/C_include>/, Regexp::MULTILINE )

  def eiffel_features( str )
    each_match( str, Eiffel_feature_regex ) { |str|
      Eiffel_feature_regex.match( str )[1]
    }
  end

  def strip_eiffel_features( str )
    str.gsub!( Eiffel_feature_regex, "" )
  end
  
  def inherit_block( str )
    each_match( str, Inherit_block_regex ) { |str|
      Inherit_block_regex.match( str )[1]
    }
  end

  def strip_inherit_block( str )
    str.gsub!( Inherit_block_regex, "" )
  end

  def c_features( str )
    each_match( str, C_feature_regex ) { |str|
      C_feature_regex.match( str )[1]
    }
  end

  def strip_c_features( str )
    str.gsub!( C_feature_regex, "" )
  end

  def c_includes( str )
    each_match( str, C_include_regex ) { |str|
      C_include_regex.match( str )[1]
    }
  end

  def strip_c_includes( str )
    str.gsub!( C_include_regex, "" )
  end

  Header_regex = Regexp.compile( /class\s*([A-Za-z_0-9]+)\s*wraps\s*([A-Za-z_0-9]+)\s*with_pointer\s*([A-Za-z_0-9]+)/ )
  Simple_header_regex = Regexp.compile( /simple_wrapper\s*([A-Za-z_0-9]+)/ )

  def class_name( str )
    m = Header_regex.match( str )
    if m != nil then 
      m[1]
    else 
      m = Simple_header_regex.match( str )
      if m != nil then
	m[1]
      else 
	nil
      end
    end
  end
 
  def wrapped_type( str )
    m = Header_regex.match( str )
    if m != nil then m[2] else nil end
  end

  def wrapping_pointer( str )
    m = Header_regex.match( str )
    if m != nil then m[3] else nil end
  end

  def strip_header( str )
    str.gsub!( Header_regex, "" )
    str.gsub!( Simple_header_regex, "" )
  end

end

class Wrapper

  Strip_eiffel_regex = Regexp.compile( /<eiffel>.*?<\/eiffel>/, Regexp::MULTILINE )
  Strip_inherit_regex = Regexp.compile( /<inherit>.*?<\/inherit>/, Regexp::MULTILINE )
  Strip_c_regex = Regexp.compile( /<C>.*?<\/C>/, Regexp::MULTILINE )

  def initialize( str )
    c = Wrapper_factory.new
    @eiffel_features = c.eiffel_features( str )
    @inherit_block = c.inherit_block( str )
    @c_features = c.c_features( str )
    @c_includes = c.c_includes( str )
    #now strip out the <eiffel> and <C> tags, because the included text can confuse the pattern-matcher
    c.strip_eiffel_features( str )
    c.strip_inherit_block( str )
    c.strip_c_features( str )
    c.strip_c_includes( str )
    #s = str.gsub( Strip_eiffel_regex, "" ).gsub( Strip_c_regex, "" ).gsub( Strip_inherit_regex, "" )
    @c_alias_structure_features = c.c_alias_structure_features_from_text( str )
    c.strip_c_alias_structure_features( str )
    @c_alias_features = c.c_alias_features_from_text( str )
    c.strip_c_alias_features( str )
    @data_features = c.data_features_from_text( str )
    c.strip_data_features( str )
    @c_with_pointer_features = c.c_with_pointer_features_from_text( str )
    c.strip_c_with_pointer_features( str )
    @c_without_pointer_features = c.c_without_pointer_features_from_text( str )
    c.strip_c_without_pointer_features( str )
    @classname = c.class_name( str )
    @wrapped_type = c.wrapped_type( str )
    @wrapping_pointer = c.wrapping_pointer( str )
    c.strip_header( str )
  end

  def data_features_texts
    eiffel_features_text = ""
    eiffel_support_features_text = ""
    c_features_text = ""
    @data_features.each { |f|
      eiffel_features_text << f.eiffel_text( @classname ) << "\n"
      eiffel_support_features_text << f.c_feature_text( @classname ) << "\n"
      c_features_text << f.c_text( @classname, @wrapping_pointer ) << "\n"
    }
    [ eiffel_features_text, eiffel_support_features_text, c_features_text ]
  end

  def c_with_pointer_features_texts
    eiffel_features_text = ""
    eiffel_support_features_text = ""
    c_features_text = ""
    @c_with_pointer_features.each { |f|
      eiffel_features_text << f.eiffel_text( @classname ) << "\n"
      eiffel_support_features_text << f.c_feature_text( @classname ) << "\n"
      c_features_text << f.c_text( @classname ) << "\n"
    }
    [ eiffel_features_text, eiffel_support_features_text, c_features_text ]
  end

  def c_without_pointer_features_texts
    eiffel_features_text = ""
    eiffel_support_features_text = ""
    c_features_text = ""
    @c_without_pointer_features.each { |f|
      eiffel_features_text << f.eiffel_text( @classname ) << "\n"
      eiffel_support_features_text << f.c_feature_text( @classname ) << "\n"
      c_features_text << f.c_text( @classname ) << "\n"
    }
    [ eiffel_features_text, eiffel_support_features_text, c_features_text ]
  end

  def c_alias_features_texts
    eiffel_features_text = ""
    eiffel_support_features_text = ""
    c_features_text = ""
    @c_alias_features.each { |f|
      eiffel_features_text << f.eiffel_text << "\n"
    }
    [ eiffel_features_text, eiffel_support_features_text, c_features_text ]
  end

  def c_alias_structure_features_texts
    eiffel_features_text = ""
    eiffel_support_features_text = ""
    c_features_text = ""
    @c_alias_structure_features.each { |f|
      eiffel_features_text << f.eiffel_text << "\n"
      eiffel_support_features_text << f.eiffel_support_alias_feature_text << "\n"
    }
    [ eiffel_features_text, eiffel_support_features_text, c_features_text ]
  end

  def eiffel_features_texts
    eiffel_features_text = ""
    @eiffel_features.each { |f|
      eiffel_features_text << f << "\n"
    }
    [ eiffel_features_text, "", "" ]
  end

  def c_features_texts
    c_features_text = ""
    @c_features.each { |f|
      c_features_text << f << "\n"
    }
    [ "", "", c_features_text]
  end

  def c_includes
    c_includes_text = ""
    @c_includes.each { |f|
      c_includes_text << f << "\n"
    }
    [ "", "", c_includes_text]
  end

  def is_structure_wrapper
    @wrapped_type != nil
  end

  def indexing_block
"
indexing
   contents: \"generated wrapper for C type #{@wrapped_type}\"
   author: \"Victor B. Putz\"
   completed: no
   tested: no
   
"
    end

  def inherit_block
    if !is_structure_wrapper then
      if @inherit_block == nil then "" else
"inherit

#{@inherit_block}
"
      end
    else
"inherit
   C_STRUCT
      redefine
	 structure_size
      end
#{@inherit_block}
"
    end
  end
    
  def creation_block
    if is_structure_wrapper then
"creation
   from_shared_memory, from_orphaned_memory, as_new_memory"
    else
      ""
    end
  end

  def eiffel_class_text
    "
#{indexing_block}
class #{@classname}
#{inherit_block}   
#{creation_block}
   
feature {ANY} -- direct Eiffel features
#{eiffel_features_texts[0]}
feature {ANY} -- structure features (aliased)
#{c_alias_structure_features_texts[0]}
feature {ANY} -- aliased helper features for above
#{c_alias_structure_features_texts[1]}
feature {ANY} -- aliased C features
#{c_alias_features_texts[0]}
feature {ANY} -- data features
#{data_features_texts[0]}
feature {ANY} -- c_with_pointer features
#{c_with_pointer_features_texts[ 0 ]}
feature {ANY} -- c_without_pointer features
#{c_without_pointer_features_texts[ 0 ]}
feature {NONE} -- data feature support functions
#{data_features_texts[1]}
feature {NONE} -- c_with_pointer feature support functions
#{c_with_pointer_features_texts[1]}
feature {NONE} -- c_without_pointer feature support functions
#{c_without_pointer_features_texts[1]}
end
"
end

def c_file_text
  "
/* Generated via make_wrapper.rb */
#{c_includes[2]}
/* data feature wrappers */
#{data_features_texts[2]}
/* c_with_pointer feature wrappers */
#{c_with_pointer_features_texts[2]}
/* c_without_pointer feature wrappers */
#{c_without_pointer_features_texts[2]} 
/* straight C features */
#{c_features_texts[2]}
"
end

end

if ARGV.length == 3
  File.open( ARGV[0], "r" ) { |infile|
    s = infile.read
    w = Wrapper.new( s )
    #write eiffel class
    File.open( ARGV[1], "w" ) { |classfile|
      classfile.write( w.eiffel_class_text )
    }
    #write C file
    File.open( ARGV[2], "w" ) { |c_file|
      c_file.write( w.c_file_text )
    }
    print "Rejected elements: #{s}"
  }
else
  print "Wrong arguments:\n\nUsage: make_wrapper interface_file eiffel_file c_file\n\n"
end


