#Takes a given header file and "sectionizes" it--placing the GC and runtime
#functions into a "runtime" section and the epalm support code into an "epalm"
#section

  class Section_designator
  
    attr_accessor :partial_pattern, :section_name

    def initialize( new_pattern, new_section_name )
      @partial_pattern = new_pattern
      @section_name = new_section_name
      @pattern = / #{@partial_pattern}\D.*?\(.*?\);/
    end

    def Section_designator.from_initializer( initializer )
      a = initializer.split( ":" )
      Section_designator.new( a[0], a[1] )
    end
    
    def matches( line )
      line =~ @pattern
    end
    
    def section_title
      "#{@section_name.upcase}_SECTION"
    end

    def substitute_in!( line )
      line.gsub!( /;/, " #{section_title};" )
    end

    def section_declaration
      "#define #{section_title} __attribute__ ((section (\"#{@section_name}\")))\n"
    end
  end


module Sectionize


  def Sectionize.sectionized_header_from_filename( filename, designators )
    lines = IO.readlines( filename )
    sectionized_header_from_lines( lines, designators )
  end

  def Sectionize.sectionize_external_functions( lines )
    lines.each { |line|
      if line =~ /^\/\*external\*\//
	#      line.gsub!( /;/, " EPALM_SECTION;" )
      end
    }
  end

  def Sectionize.sectionize_builtin_types( lines )
    count = 0
    lines.each { |line|
      if line =~ / r[1-8]\D.*?\(.*?\);/
	line.gsub!( /;/, " BUILTIN_TYPES_SECTION;" )
	count = count+1
      end
    }
    print "btypes: #{count}\n"
  end

  def Sectionize.sectionize_gc_functions( lines )
    count1 = 0
    count2 = 0
    count3 = 0
    count4 = 0
    lines.each { |line|
      if ( line =~ / gc_sweep.*?\(.*?\);/ ) 
	line.gsub!( /;/, " GC1_SECTION;" ) 
	count1 = count1+1
      elsif ( line =~ / gc_mark.*?\(.*?\);/ )
	line.gsub!( /;/, " GC2_SECTION;" )
	count2 = count2+1
      elsif line =~ ( /[\* ]new.*?\(.*?\);/ ) 
	line.gsub!( /;/, " GC3_SECTION;" )
	count3 = count3 + 1
      elsif line =~( / gc_align_mark.*?\(.*?\);/ )
	line.gsub!( /;/, " GC4_SECTION;" )
	count4 = count4 + 1
      end
    }
    print "gc1: #{count1}\n"
    print "gc2: #{count2}\n"
    print "gc3: #{count3}\n"
    print "gc4: #{count4}\n"
  end

  def Sectionize.sectionize_with_designators( lines, designators )
    counters = {}
    lines.each { |line|
      designators.each {|designator|
	if designator.matches( line )
	  designator.substitute_in!( line )
	  if counters.has_key?( designator )
	    counters[designator]= counters[designator]+1
	  else
	    counters[designator] = 1
	  end
	end
      }
    }
    counters.each { |d, count|
      print "#{d.section_name}: #{count}\n"
    }
  end

  def Sectionize.designator_section_declarations( designators )
    result = []
    designators.each { |designator|
      result << designator.section_declaration
    }
    result
  end
  
  def Sectionize.is_sectionized( lines )
#    return lines.include?("#define BUILTIN_TYPES_SECTION __attribute__ ((section (\"btypes\")))")
    return lines[0].index( "BUILTIN_TYPES_SECTION" ) != nil
  end

  def Sectionize.sectionized_header_from_lines( lines, designators )
    # we are given an array of lines which may or may not be sectionized.
    # before we sectionize it, we must check to see if it is already 
    # sectionized!
    if is_sectionized( lines )
      print "Already sectionized\n"
    else
      print "Sectionizing\n"
      sectionize_gc_functions( lines )
      sectionize_builtin_types( lines )
      sectionize_external_functions( lines )
      sectionize_with_designators( lines, designators )
      result = Sectionize.header() + designator_section_declarations( designators ) + lines
      lines = result
    end
    lines
  end

  def Sectionize.header()
    result = [
      "#define BUILTIN_TYPES_SECTION __attribute__ ((section (\"btypes\")))\n",
      "#define GC1_SECTION __attribute__ ((section (\"gc1\")))\n",
      "#define GC2_SECTION __attribute__ ((section (\"gc2\")))\n",
      "#define GC3_SECTION __attribute__ ((section (\"gc3\")))\n",
      "#define GC4_SECTION __attribute__ ((section (\"gc4\")))\n" ]
    return result
  end
  
  def Sectionize.sectionize_file( filename, designators )
    newlines = sectionized_header_from_filename( filename, designators )
    File.open( filename, "w" ) {
      |outfile|
      #    outfile.write( "#define EPALM_SECTION __attribute__ ((section (\"epalm\")))\n" )
      #outfile.write( "#define BUILTIN_TYPES_SECTION __attribute__ ((section (\"btypes\")))\n" )
      #outfile.write( "#define GC1_SECTION __attribute__ ((section (\"gc1\")))\n" )
      #outfile.write( "#define GC2_SECTION __attribute__ ((section (\"gc2\")))\n" )
      #outfile.write( "#define GC3_SECTION __attribute__ ((section (\"gc3\")))\n" )
      #outfile.write( "#define GC4_SECTION __attribute__ ((section (\"gc4\")))\n" )
      outfile.write( designator_section_declarations( designators ) )
      outfile.write( newlines )
    }
  end    

  
end

