def c_text_from_filename (filename) 
  IO.readlines( filename ).join
end

C_plus_plus_open = Regexp.compile( "#ifdef __cplusplus\nextern \"C\" {\n#endif\n", Regexp::MULTILINE )
C_plus_plus_close = Regexp.compile( "#ifdef __cplusplus\n}\n#endif\n", Regexp::MULTILINE )

def text_minus_extern_c (text)
 text.gsub( C_plus_plus_close, "" ).gsub( C_plus_plus_open, "" )
#  text.gsub( C_plus_plus_open, "" )
end

def include_directive( c_filename ) 
  include_filename = c_filename.gsub( /\.c/, ".h" )
  "#include \"#{include_filename}\"\n"
end

def text_minus_include (text, c_filename)
  r = Regexp.compile( include_directive( c_filename ) )
  text.gsub( r, "" )
end

Separator = "\n}\n\n"

def stripped_text_from_filename( filename )
  text_minus_include( text_minus_extern_c( c_text_from_filename( filename ) ), filename )
end

def c_text_sections_from_filename (filename)
  stripped_text_from_filename( filename ).split( Separator )
end

def first_block(sections, block_size)
  [sections[0..block_size].join( Separator) + Separator]
end

def remainder( sections, block_size )
  sections[block_size + 1..-1]
end

def num_blocks_for_size( sections, block_size_in_bytes )
  i = 0
  size_accumulator = 0
  sections.each { |section|
    if ( size_accumulator < block_size_in_bytes ) then
      i = i + 1
      size_accumulator = size_accumulator + section.length
    end
  }
  i
end

def join_in_blocks( sections, block_size_in_bytes )
  block_size_in_sections = num_blocks_for_size( sections, block_size_in_bytes )
  if sections.length <= block_size_in_sections
    [sections.join( Separator ) + Separator]
  else
    first_block( sections, block_size_in_sections ) + join_in_blocks( remainder(sections, block_size_in_sections ), block_size_in_bytes )
  end
end

def split_c_source( filename, num_blocks )
  raw_text = stripped_text_from_filename( filename )
  block_size_in_bytes = raw_text.length / num_blocks

  a = c_text_sections_from_filename( filename )
  join_in_blocks( a, block_size_in_bytes )
end

def create_split_c_files( filename, num_blocks )
  blocks = split_c_source( filename, num_blocks )
  i = 0
  blocks.each { |block|
    i = i + 1
    split_filename = filename.gsub( /\.c/, "#{i}.c" )
    File.open( split_filename, "w" ) { |file|
      file.write( include_directive( filename ) )
      file.write( block )
    }
  }
end

if ARGV.length > 0 then
  create_split_c_files( ARGV[0], ARGV[1].to_i )
else
  print "\nUsage: \n\tsplit_c filename num_sections\n"
end
